package org.jpedal.examples.simpleviewer;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.GraphicsEnvironment;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.TextArea;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.image.BufferedImage;
import java.awt.print.PageFormat;
import java.awt.print.Paper;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.Set;
import java.util.StringTokenizer;
import javax.imageio.ImageIO;
import javax.media.jai.JAI;
import javax.media.jai.RenderedOp;
import javax.swing.AbstractButton;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.JToolBar;
import javax.swing.JTree;
import javax.swing.JViewport;
import javax.swing.RepaintManager;
import javax.swing.SwingUtilities;
import javax.swing.Timer;
import javax.swing.border.EtchedBorder;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.text.BadLocationException;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;
import org.jpedal.PdfDecoder;
import org.jpedal.examples.simpleviewer.gui.SearchList;
import org.jpedal.examples.simpleviewer.gui.ThumbnailPanel;
import org.jpedal.examples.simpleviewer.utils.FileFilterer;
import org.jpedal.examples.simpleviewer.utils.FormActionListener;
import org.jpedal.examples.simpleviewer.utils.IconiseImage;
import org.jpedal.examples.simpleviewer.utils.SwingWorker;
import org.jpedal.exception.PdfException;
import org.jpedal.exception.PdfFontException;
import org.jpedal.grouping.PdfGroupingAlgorithms;
import org.jpedal.io.StatusBar;
import org.jpedal.objects.PdfAnnots;
import org.jpedal.objects.PdfFileInformation;
import org.jpedal.objects.PdfPageData;
import org.jpedal.objects.acroforms.AcroRenderer;
import org.jpedal.utils.LogWriter;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class SimpleViewer extends JFrame
{
    protected class FrameCloser extends WindowAdapter
    {

        public void windowClosing(WindowEvent windowevent)
        {
            if(target != null)
                flush();
            if(SimpleViewer.printingThreads > 0)
                JOptionPane.showMessageDialog(c, getMessage("PdfViewerBusyPrinting.message"));
            if(!isProcessing)
            {
                if(showThumbnails && thumbnails.drawing)
                {
                    thumbnails.interrupt = true;
                    while(thumbnails.drawing) 
                        try
                        {
                            Thread.sleep(20L);
                        }
                        catch(InterruptedException interruptedexception)
                        {
                            interruptedexception.printStackTrace();
                        }
                    thumbnails.interrupt = false;
                }
                int i = JOptionPane.showConfirmDialog(c, getMessage("PdfViewerCloseing.message"), null, 0);
                if(i == 0)
                {
                    decode_pdf.closePdfFile();
                    System.exit(0);
                }
            } else
            {
                JOptionPane.showMessageDialog(c, getMessage("PdfViewerDecodeWait.message"));
            }
        }

        public FrameCloser()
        {
        }
    }

    protected class mouse_clicker extends MouseAdapter
    {

        public void mousePressed(MouseEvent mouseevent)
        {
            int i = (int)((float)(mouseevent.getX() - 25) / scaling);
            int j = (int)((float)(mouseevent.getY() - 25) / scaling);
            if(maxViewY != 0)
            {
                i = (int)((double)((float)i - (float)dx * scaling) / viewportScale);
                j = (int)(((double)mediaH - (double)((float)mediaH - (float)j / scaling - (float)dy) / viewportScale) * (double)scaling);
            }
            if(rotation == 90)
            {
                m_y1 = i + cropY;
                m_x1 = j + cropX;
            } else
            if(rotation == 180)
            {
                m_x1 = mediaW - ((i + mediaW) - cropW - cropX);
                m_y1 = j + cropY;
            } else
            if(rotation == 270)
            {
                m_y1 = mediaH - ((i + mediaH) - cropH - cropY);
                m_x1 = mediaW - ((j + mediaW) - cropW - cropX);
            } else
            {
                m_x1 = i + cropX;
                m_y1 = mediaH - ((j + mediaH) - cropH - cropY);
            }
            updateCords(mouseevent);
        }

        public void mouseClicked(MouseEvent mouseevent)
        {
            if(!decode_pdf.showAnnotations)
                checkLinks(true);
        }

        public void mouseReleased(MouseEvent mouseevent)
        {
            old_m_x2 = -1;
            old_m_y2 = -1;
            updateCords(mouseevent);
            if(isPDF && mouseevent.isShiftDown())
                extractSelectedScreenAsImage();
            decode_pdf.updateCursorBoxOnScreen(null, null);
            decode_pdf.setHighlightedAreas(null);
            decode_pdf.repaintArea(new Rectangle(m_x1 - cropX, m_y2 + cropY, (m_x2 - m_x1) + cropX, (m_y1 - m_y2) + cropY), mediaH);
            decode_pdf.repaint();
        }

        protected mouse_clicker()
        {
        }
    }

    protected class mouse_mover
        implements MouseMotionListener
    {

        public void mouseDragged(MouseEvent mouseevent)
        {
            int ai[] = updateXY(mouseevent);
            m_x2 = ai[0];
            m_y2 = ai[1];
            scrollAndUpdateCoords(mouseevent);
            generateNewCursorBox();
            if(!decode_pdf.showAnnotations)
                checkLinks(false);
        }

        protected void scrollAndUpdateCoords(MouseEvent mouseevent)
        {
            int i = decode_pdf.getScrollInterval();
            Rectangle rectangle = new Rectangle(mouseevent.getX(), mouseevent.getY(), i, i);
            if(allowScrolling && !decode_pdf.getVisibleRect().contains(rectangle))
                decode_pdf.scrollRectToVisible(rectangle);
            updateCords(mouseevent);
        }

        protected void generateNewCursorBox()
        {
            if((old_m_x2 != -1) | (old_m_y2 != -1) | (Math.abs(m_x2 - old_m_x2) > 5) | (Math.abs(m_y2 - old_m_y2) > 5))
            {
                int i = m_x1;
                if(m_x1 > m_x2)
                    i = m_x2;
                int j = m_y1;
                if(m_y1 > m_y2)
                    j = m_y2;
                int k = Math.abs(m_x2 - m_x1);
                int l = Math.abs(m_y2 - m_y1);
                currentRectangle = new Rectangle(i, j, k, l);
                decode_pdf.updateCursorBoxOnScreen(currentRectangle, Color.blue);
                Rectangle arectangle[] = new Rectangle[2];
                arectangle[0] = currentRectangle;
                decode_pdf.setHighlightedAreas(arectangle);
                old_m_x2 = m_x2;
                old_m_y2 = m_y2;
            }
        }

        public void mouseMoved(MouseEvent mouseevent)
        {
            updateCords(mouseevent);
            if(!decode_pdf.showAnnotations)
                checkLinks(false);
        }

        public mouse_mover()
        {
        }
    }

    class PageChanger
        implements ActionListener
    {

        public void actionPerformed(ActionEvent actionevent)
        {
            if(!isProcessing && currentPage != page)
            {
                currentPage = page;
                statusBar.resetStatus("");
                scalingBox.setSelectedIndex(0);
                decode_pdf.setPageParameters(scaling, currentPage);
                decodePage(false);
            }
        }

        int page;

        public PageChanger(int i)
        {
            i++;
            page = i;
        }
    }

    private class ThumbPainter extends ComponentAdapter
    {

        public void componentMoved(ComponentEvent componentevent)
        {
            if(trapMultipleMoves.isRunning())
                trapMultipleMoves.stop();
            trapMultipleMoves.setRepeats(false);
            trapMultipleMoves.start();
        }

        Timer trapMultipleMoves;


        private ThumbPainter()
        {
            trapMultipleMoves = trapMultipleMoves = new Timer(250, new ActionListener() {

                public void actionPerformed(ActionEvent actionevent)
                {
                    if(worker != null && thumbnails.drawing)
                    {
                        thumbnails.interrupt = true;
                        while(thumbnails.drawing) 
                            try
                            {
                                Thread.sleep(20L);
                            }
                            catch(InterruptedException interruptedexception)
                            {
                                interruptedexception.printStackTrace();
                            }
                        thumbnails.interrupt = false;
                    }
                    while(isProcessing) 
                        try
                        {
                            Thread.sleep(20L);
                        }
                        catch(InterruptedException interruptedexception1)
                        {
                            interruptedexception1.printStackTrace();
                        }
                    thumbnails.drawVisibleThumbnailsOnScroll(decode_pdf);
                }

            });
        }

    }


    public void setupViewer(String s)
    {
        setupViewer();
        openDefaultFile(s);
    }

    protected void openDefaultFile(String s)
    {
        if(s != null)
        {
            File file = new File(s);
            if(!file.exists())
                JOptionPane.showMessageDialog(this, s + "\n" + getMessage("PdfViewerdoesNotExist.message"));
            else
            if(file.isDirectory())
            {
                JOptionPane.showMessageDialog(this, s + "\n" + getMessage("PdfViewerFileIsDirectory.message"));
            } else
            {
                selectedFile = s;
                setViewerTitle(null);
                size = file.length() >> 10;
                String s1 = System.getProperty("Page");
                String s2 = System.getProperty("Bookmark");
                if(s1 != null)
                {
                    byte byte0 = -1;
                    try
                    {
                        int i = Integer.parseInt(s1);
                        if(i < 1)
                        {
                            i = -1;
                            System.err.println(s1 + " must be 1 or larger. Opening on page 1");
                            LogWriter.writeLog(s1 + " must be 1 or larger. Opening on page 1");
                        }
                        if(i != -1)
                            openFile(file, i);
                    }
                    catch(Exception exception)
                    {
                        System.err.println(s1 + "is not a valid number for a page number. Opening on page 1");
                        LogWriter.writeLog(s1 + "is not a valid number for a page number. Opening on page 1");
                    }
                } else
                if(s2 != null)
                    openFile(file, s2);
                else
                    openFile(s);
            }
            s = null;
        }
    }

    public SimpleViewer()
    {
        textPages = new HashMap();
        textRectangles = new HashMap();
        display = true;
        switchModes = true;
        showThumbnailsdefault = false;
        showThumbnails = showThumbnailsdefault;
        thumbnails = new ThumbnailPanel();
        bookmarkScrollPane = new JScrollPane();
        navOptionsPanel = new JTabbedPane();
        isPDF = true;
        firstTime = true;
        titleMessage = null;
        pageLookupTable = new HashMap();
        pointLookupTable = new HashMap();
        ignoreAlteredBookmark = false;
        inputDir = System.getProperty("user.dir");
        currentPage = 1;
        rotation = 0;
        old_m_x2 = -1;
        old_m_y2 = -1;
        currentRectangle = null;
        worker = null;
        searcher = null;
        isSearch = false;
        searchButton = new JButton("search");
        pageCount = 1;
        scaling = 1.0F;
        selectedFile = null;
        isProcessing = false;
        scrollPane = new JScrollPane();
        rotationBox = new JComboBox(rotationValues);
        pageCounter2 = new JTextField(4);
        messages = null;
        img = null;
        message = "";
        first = new JButton();
        fback = new JButton();
        back = new JButton();
        forward = new JButton();
        fforward = new JButton();
        end = new JButton();
        allowScrolling = true;
        hasOutlinesDrawn = false;
        bookmarksButton = new JButton();
        statusBar = new StatusBar(Color.orange);
        divLocation = 170;
        thumbLocation = 200;
        minimumScreenWidth = 800;
        dy = 0;
        viewportScale = 1.0D;
        maxViewY = 0;
        painter = new ThumbPainter();
        showOutlines = true;
        itemFoundCount = 0;
        pagesDecoded = new HashMap();
    }

    public final void flush()
    {
        if(target != null)
        {
            File file = new File(target);
            String as[] = file.list();
            if(as != null)
            {
                for(int i = 0; i < as.length; i++)
                {
                    File file1 = new File(target + separator + as[i]);
                    file1.delete();
                }

            }
        }
    }

    public void setupViewer()
    {
        String s = System.getProperty("thumbnail");
        if(s != null && s.equals("true"))
        {
            showThumbnailsdefault = true;
            showThumbnails = true;
        }
        setupGUI();
    }

    protected void init(ResourceBundle resourcebundle)
    {
        if(resourcebundle == null)
            try
            {
                bundle = ResourceBundle.getBundle("org.jpedal.international.messages");
            }
            catch(Exception exception)
            {
                exception.printStackTrace();
                System.out.println("Exception loading resource bundle");
            }
        else
            bundle = resourcebundle;
        decode_pdf = new PdfDecoder();
        decode_pdf.init(true);
        decode_pdf.setInset(25, 25);
        decode_pdf.setStatusBarObject(statusBar);
        if(!decode_pdf.showAnnotations)
            decode_pdf.createPageHostspots(annotTypes, "org/jpedal/examples/simpleviewer/annots/");
        try
        {
            decode_pdf.setDefaultDisplayFont("SansSerif");
        }
        catch(PdfFontException pdffontexception)
        {
            System.out.println(pdffontexception.getMessage());
            String as[] = GraphicsEnvironment.getLocalGraphicsEnvironment().getAvailableFontFamilyNames();
            System.out.println(getMessage("PdfViewerFontsFound.message"));
            System.out.println("=====================\n");
            int i = as.length;
            for(int j = 0; j < i; j++)
            {
                Font font = new Font(as[j], 1, 10);
                System.out.println(as[j] + " (" + getMessage("PdfViewerFontsPostscript.message") + "=" + font.getPSName() + ")");
            }

            System.exit(1);
        }
        c = getContentPane();
        c.setLayout(new BorderLayout());
        RepaintManager repaintmanager = RepaintManager.currentManager(this);
        repaintmanager.setDoubleBufferingEnabled(true);
        scrollPane.getViewport().add(decode_pdf);
        scrollPane.setVerticalScrollBarPolicy(20);
        scrollPane.setHorizontalScrollBarPolicy(30);
        scrollPane.getVerticalScrollBar().setUnitIncrement(80);
        scrollPane.getHorizontalScrollBar().setUnitIncrement(80);
        bookmarkScrollPane.setHorizontalScrollBarPolicy(30);
        bookmarkScrollPane.setVerticalScrollBarPolicy(20);
        navOptionsPanel.setTabPlacement(1);
        displayPane = new JSplitPane(1, navOptionsPanel, scrollPane);
        displayPane.setOneTouchExpandable(true);
        c.add(displayPane, "Center");
    }

    private void extractSelectedScreenAsImage()
    {
        int i = m_x1;
        int j = m_x2;
        int k = m_y1;
        int l = m_y2;
        if(m_y1 < m_y2)
        {
            l = m_y1;
            k = m_y2;
        }
        if(m_x1 > m_x2)
        {
            j = m_x1;
            i = m_x2;
        }
        final BufferedImage snapShot = decode_pdf.getSelectedRectangleOnscreen(i, k, j, l, 100);
        JPanel jpanel = new JPanel();
        jpanel.setLayout(new BorderLayout());
        if(snapShot != null)
        {
            IconiseImage iconiseimage = new IconiseImage(snapShot);
            jpanel.add(new JLabel(iconiseimage), "Center");
        } else
        {
            return;
        }
        final JScrollPane image_scroll = new JScrollPane();
        image_scroll.getViewport().add(jpanel);
        final JDialog displayFrame = new JDialog(this, true);
        int i1 = snapShot.getWidth();
        if(i1 < snapShot.getHeight())
            i1 = snapShot.getHeight();
        i1 += 50;
        if(i1 > 450)
            i1 = 450;
        displayFrame.setLocationRelativeTo(null);
        displayFrame.setLocation(getLocationOnScreen().x + 10, getLocationOnScreen().y + 10);
        displayFrame.setSize(i1, i1);
        displayFrame.setTitle("Extracted Image - would you like to save?");
        displayFrame.getContentPane().setLayout(new BorderLayout());
        displayFrame.getContentPane().add(image_scroll, "Center");
        JPanel jpanel1 = new JPanel();
        jpanel1.setLayout(new BorderLayout());
        displayFrame.getContentPane().add(jpanel1, "South");
        JButton jbutton = new JButton("YES");
        jbutton.setFont(new Font("SansSerif", 0, 12));
        jpanel1.add(jbutton, "West");
        jbutton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent actionevent)
            {
                displayFrame.hide();
                JFileChooser jfilechooser = new JFileChooser(System.getProperty("user.dir"));
                String as[] = {
                    "png"
                };
                String as1[] = {
                    "tif", "tiff"
                };
                String as2[] = {
                    "jpg", "jpeg"
                };
                jfilechooser.addChoosableFileFilter(new FileFilterer(as1, "TIFF"));
                jfilechooser.addChoosableFileFilter(new FileFilterer(as2, "JPEG"));
                int j1 = jfilechooser.showSaveDialog(image_scroll);
                if(j1 == 0)
                {
                    File file = jfilechooser.getSelectedFile();
                    String s = jfilechooser.getFileFilter().getDescription();
                    if(s.equals("All Files"))
                        s = "TIFF";
                    String s1 = file.getPath() + "." + s;
                    if(snapShot != null)
                        JAI.create("filestore", snapShot, s1, s);
                    displayFrame.dispose();
                }
            }

        });
        JButton jbutton1 = new JButton("NO");
        jbutton1.setFont(new Font("SansSerif", 0, 12));
        jpanel1.add(jbutton1, "East");
        jbutton1.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent actionevent)
            {
                displayFrame.dispose();
            }

        });
        displayFrame.show();
    }

    private void createUniqueAnnontationIcons()
    {
        byte byte0 = 20;
        byte byte1 = 20;
        int i = 1;
        for(int j = 1; j < byte0 + 1; j++)
        {
            Image aimage[] = new Image[byte1];
            for(int k = 0; k < byte1; k++)
            {
                aimage[k] = new BufferedImage(32, 32, 2);
                Graphics2D graphics2d = (Graphics2D)aimage[k].getGraphics();
                graphics2d.setColor(Color.black);
                graphics2d.fill(new Rectangle(0, 0, 32, 32));
                graphics2d.setColor(Color.white);
                graphics2d.draw(new Rectangle(0, 0, 32, 32));
                graphics2d.setColor(Color.white);
                graphics2d.drawString(j + "/" + k + annotTypes[i], 0, 10);
            }

            decode_pdf.addUserIconsForAnnotations(j, annotTypes[i], aimage);
        }

    }

    private void readChildNodes(Node node, DefaultMutableTreeNode defaultmutabletreenode)
    {
        NodeList nodelist = node.getChildNodes();
        int i = nodelist.getLength();
        for(int j = 0; j < i; j++)
        {
            Node node1 = nodelist.item(j);
            Element element = (Element)node1;
            String s = element.getAttribute("title");
            String s1 = element.getAttribute("page");
            String s2 = element.getAttribute("Dest");
            String s3 = element.getAttribute("objectRef");
            pageLookupTable.put(s, s1);
            if(s2 != null && s2.indexOf("/XYZ") != -1)
            {
                s2 = s2.substring(s2.indexOf("/XYZ") + 4);
                StringTokenizer stringtokenizer = new StringTokenizer(s2, "[] ");
                String s4 = stringtokenizer.nextToken();
                if(s4.equals("null"))
                    s4 = "0";
                String s5 = stringtokenizer.nextToken();
                if(s5.equals("null"))
                    s5 = "0";
                pointLookupTable.put(s, new Point((int)Float.parseFloat(s4), (int)Float.parseFloat(s5)));
            }
            DefaultMutableTreeNode defaultmutabletreenode1 = new DefaultMutableTreeNode(s);
            defaultmutabletreenode.add(defaultmutabletreenode1);
            if(node1.hasChildNodes())
                readChildNodes(node1, defaultmutabletreenode1);
        }

    }

    private void expandAll()
    {
        for(int i = 0; i < tree.getRowCount(); i++)
            tree.expandRow(i);

    }

    private void createOutlinePanels()
    {
        boolean flag = false;
        if(showThumbnails)
        {
            int i = decode_pdf.getPageCount();
            if(i > 100)
            {
                showThumbnails = false;
                LogWriter.writeLog("Thumbnails not used on files over 100 pages long");
            } else
            {
                flag = true;
                navOptionsPanel.add(thumbnails.setupThumbnails(i, textFont, getMessage("PdfViewerPageLabel.text"), decode_pdf.getPdfPageData()), getMessage("PdfViewerTitle.thumbnails"));
                for(int j = 0; j < i; j++)
                    thumbnails.getButton(j).addActionListener(new PageChanger(j));

                thumbnails.addComponentListener(painter);
            }
        }
        if(decode_pdf.hasOutline() && showOutlines)
        {
            flag = true;
            DefaultMutableTreeNode defaultmutabletreenode = new DefaultMutableTreeNode("Root");
            Node node = decode_pdf.getOutlineAsXML().getFirstChild();
            if(node != null)
            {
                readChildNodes(node, defaultmutabletreenode);
                tree = new JTree(defaultmutabletreenode);
                expandAll();
                tree.setRootVisible(false);
                bookmarkScrollPane.getViewport().add(tree);
                navOptionsPanel.add(bookmarkScrollPane, getMessage("PdfViewerTitle.bookmarks"));
                tree.addTreeSelectionListener(new TreeSelectionListener() {

                    public void valueChanged(TreeSelectionEvent treeselectionevent)
                    {
                        if(ignoreAlteredBookmark)
                            return;
                        DefaultMutableTreeNode defaultmutabletreenode1 = (DefaultMutableTreeNode)tree.getLastSelectedPathComponent();
                        if(defaultmutabletreenode1 == null)
                            return;
                        Object obj = defaultmutabletreenode1.getUserObject();
                        String s = (String)defaultmutabletreenode1.getUserObject();
                        String s1 = (String)pageLookupTable.get(s);
                        if(s1 != null && s1.length() > 0)
                        {
                            int k = Integer.parseInt(s1);
                            if(!isProcessing && currentPage != k)
                            {
                                currentPage = k;
                                scalingBox.setSelectedIndex(0);
                                decode_pdf.setPageParameters(scaling, currentPage);
                                decodePage(false);
                            }
                            Point point = (Point)pointLookupTable.get(s);
                            if(point != null)
                                decode_pdf.ensurePointIsVisible(point);
                        } else
                        {
                            System.out.println("No dest page set for " + s);
                        }
                    }

                });
                tree.getSelectionModel().setSelectionMode(1);
            } else
            {
                flag = false;
                bookmarksButton.setEnabled(false);
            }
        }
        if(flag && !showThumbnails)
        {
            navOptionsPanel.setVisible(true);
            displayPane.setDividerLocation(divLocation);
            displayPane.invalidate();
            displayPane.repaint();
        }
    }

    private void removeOutlinePanels()
    {
        thumbnails.removeAll();
        bookmarkScrollPane.setMinimumSize(new Dimension(50, getHeight()));
        if(!showThumbnails)
            navOptionsPanel.setVisible(false);
        navOptionsPanel.removeAll();
        hasOutlinesDrawn = false;
        bookmarksButton.setEnabled(decode_pdf.hasOutline());
    }

    protected int[] updateXY(MouseEvent mouseevent)
    {
        x = (int)((float)(mouseevent.getX() - 25) / scaling);
        y = (int)((float)(mouseevent.getY() - 25) / scaling);
        if(maxViewY != 0)
        {
            x = (int)((double)((float)x - (float)dx * scaling) / viewportScale);
            y = (int)(((double)mediaH - (double)((float)mediaH - (float)y / scaling - (float)dy) / viewportScale) * (double)scaling);
        }
        int ai[] = new int[2];
        if(rotation == 90)
        {
            ai[1] = x + cropY;
            ai[0] = y + cropX;
        } else
        if(rotation == 180)
        {
            ai[0] = mediaW - ((x + mediaW) - cropW - cropX);
            ai[1] = y + cropY;
        } else
        if(rotation == 270)
        {
            ai[1] = mediaH - ((x + mediaH) - cropH - cropY);
            ai[0] = mediaW - ((y + mediaW) - cropW - cropX);
        } else
        {
            ai[0] = x + cropX;
            ai[1] = mediaH - ((y + mediaH) - cropH - cropY);
        }
        return ai;
    }

    public void checkLinks(boolean flag)
    {
        message = "";
        Rectangle arectangle[] = decode_pdf.getPageHotspots();
        if(arectangle != null)
        {
            int i = arectangle.length;
            int j = -1;
            for(int k = 0; k < i; k++)
                if(arectangle[k] != null && arectangle[k].contains(cx, cy))
                {
                    j = k;
                    k = i;
                }

            if(j != -1)
            {
                flag = false;
                if(flag)
                {
                    Map map = pageAnnotations.getAnnotRawData(j);
                    Map map1 = decode_pdf.resolveToMapOrString("A", map);
                    String s = pageAnnotations.getAnnotSubType(j);
                    if(s.equals("Link") && map1 != null)
                    {
                        Iterator iterator = map1.keySet().iterator();
                        JPanel jpanel1 = new JPanel();
                        jpanel1.setLayout(new BoxLayout(jpanel1, 1));
                        String s3;
                        for(; iterator.hasNext(); jpanel1.add(new JLabel(s3 + " : " + decode_pdf.resolveToMapOrString(s3, map))))
                            s3 = (String)iterator.next();

                        JOptionPane.showMessageDialog(this, jpanel1, getMessage("PdfViewerTitle.annots"), -1);
                    } else
                    if(s.equals("Text"))
                    {
                        String s1 = pageAnnotations.getField(j, "T");
                        if(s1 == null)
                            s1 = getMessage("PdfViewerAnnots.notitle");
                        String s2 = pageAnnotations.getField(j, "Contents");
                        if(s2 == null)
                            s2 = getMessage("PdfViewerAnnots.nocont");
                        JOptionPane.showMessageDialog(this, new TextArea(s2), s1, -1);
                    } else
                    if(s.equals("FileAttachment"))
                    {
                        Map map2 = decode_pdf.resolveToMapOrString("FS", map);
                        if(map2 != null)
                            map2 = (Map)map2.get("EF");
                        if(map2 != null)
                            map2 = (Map)map2.get("F");
                        if(map2 != null)
                        {
                            byte abyte0[] = (byte[])map2.get("DecodedStream");
                            if(abyte0 == null)
                            {
                                JOptionPane.showMessageDialog(this, getMessage("PdfViewerAnnots.nofile"));
                            } else
                            {
                                JFileChooser jfilechooser = new JFileChooser(inputDir);
                                jfilechooser.setFileSelectionMode(0);
                                int l = jfilechooser.showSaveDialog(this);
                                if(l == 0)
                                {
                                    File file = jfilechooser.getSelectedFile();
                                    try
                                    {
                                        FileOutputStream fileoutputstream = new FileOutputStream(file);
                                        fileoutputstream.write(abyte0);
                                        fileoutputstream.close();
                                    }
                                    catch(Exception exception)
                                    {
                                        exception.printStackTrace();
                                    }
                                }
                            }
                        }
                    } else
                    {
                        JPanel jpanel = new JPanel();
                        jpanel.setLayout(new BoxLayout(jpanel, 1));
                        String s4;
                        for(Iterator iterator1 = ((Map)map).keySet().iterator(); iterator1.hasNext(); jpanel.add(new JLabel(s4 + " : " + decode_pdf.resolveToMapOrString(s4, map))))
                            s4 = (String)iterator1.next();

                        JOptionPane.showMessageDialog(this, jpanel, getMessage("PdfViewerAnnots.nosubtype") + " " + s, -1);
                    }
                } else
                {
                    message = getMessage("PdfViewerAnnots.entered") + " " + j;
                }
            }
        }
    }

    protected final JScrollPane createPane(JTextPane jtextpane, String s, boolean flag)
        throws BadLocationException
    {
        jtextpane.setEditable(true);
        jtextpane.setFont(new Font("Lucida", 0, 14));
        jtextpane.setToolTipText(getMessage("PdfViewerTooltip.text"));
        javax.swing.text.Document document = jtextpane.getDocument();
        jtextpane.setBorder(BorderFactory.createTitledBorder(new EtchedBorder(), getMessage("PdfViewerTitle.text")));
        jtextpane.setForeground(Color.black);
        SimpleAttributeSet simpleattributeset = new SimpleAttributeSet();
        SimpleAttributeSet simpleattributeset1 = new SimpleAttributeSet();
        SimpleAttributeSet simpleattributeset2 = new SimpleAttributeSet();
        StyleConstants.setForeground(simpleattributeset, Color.blue);
        StyleConstants.setForeground(simpleattributeset1, Color.black);
        StyleConstants.setForeground(simpleattributeset2, Color.black);
        int i = 0;
        if(flag && s != null)
        {
            for(StringTokenizer stringtokenizer = new StringTokenizer(s, "<>", true); stringtokenizer.hasMoreTokens();)
            {
                String s1 = stringtokenizer.nextToken();
                if(s1.equals("<") && stringtokenizer.hasMoreTokens())
                {
                    String s2 = s1 + stringtokenizer.nextToken() + stringtokenizer.nextToken();
                    document.insertString(i, s2, simpleattributeset);
                    i += s2.length();
                } else
                {
                    document.insertString(i, s1, simpleattributeset1);
                    i += s1.length();
                }
            }

        } else
        {
            document.insertString(i, s, simpleattributeset2);
        }
        JScrollPane jscrollpane = new JScrollPane();
        jscrollpane.getViewport().add(jtextpane);
        jscrollpane.setHorizontalScrollBarPolicy(31);
        jscrollpane.setVerticalScrollBarPolicy(20);
        return jscrollpane;
    }

    public void updateCords(MouseEvent mouseevent)
    {
        int i = mouseevent.getX() - 25;
        int j = mouseevent.getY() - 25;
        if(maxViewY != 0)
        {
            i = (int)((double)((float)i - (float)dx * scaling) / viewportScale);
            j = (int)(((double)mediaH - (double)((float)mediaH - (float)j / scaling - (float)dy) / viewportScale) * (double)scaling);
        }
        cx = (int)((float)i / scaling);
        cy = (int)((float)j / scaling);
        if(rotation == 90)
        {
            int k = cx + cropY;
            cx = cy + cropX;
            cy = k;
        } else
        if(rotation == 180)
        {
            cx = mediaW - ((cx + mediaW) - cropW - cropX);
            cy = cy + cropY;
        } else
        if(rotation == 270)
        {
            int l = mediaH - ((cx + mediaH) - cropH - cropY);
            cx = mediaW - ((cy + mediaW) - cropW - cropX);
            cy = l;
        } else
        {
            cx = cx + cropX;
            cy = mediaH - ((cy + mediaH) - cropH - cropY);
        }
        if(isProcessing | (selectedFile == null))
            coords.setText("  X:  Y:   ");
        else
            coords.setText("  X: " + cx + " Y: " + cy + " " + " " + message);
        if(allowScrolling && !mouseevent.isShiftDown())
        {
            int i1 = decode_pdf.getScrollInterval() * 2;
            Rectangle rectangle = new Rectangle(mouseevent.getX() - i1, mouseevent.getY() - i1, i1 * 2, i1 * 2);
            if(!decode_pdf.getVisibleRect().contains(rectangle))
                decode_pdf.scrollRectToVisible(rectangle);
        }
    }

    protected void setupGUI()
    {
        init(null);
        setViewerTitle(getMessage("PdfViewerCom.message") + "  " + "FRF-2.60b10");
        initGUIComponents();
        JPanel jpanel = new JPanel();
        jpanel.setLayout(new BorderLayout());
        getContentPane().add(jpanel, "North");
        decode_pdf.addMouseMotionListener(new mouse_mover());
        decode_pdf.addMouseListener(new mouse_clicker());
        JMenuBar jmenubar = new JMenuBar();
        jpanel.add(jmenubar, "North");
        createMenu(jmenubar, true);
        JToolBar jtoolbar = new JToolBar();
        jtoolbar.setBorder(BorderFactory.createEmptyBorder());
        jtoolbar.setLayout(new FlowLayout(3));
        jtoolbar.setFloatable(false);
        jtoolbar.setFont(new Font("SansSerif", 0, 8));
        jpanel.add(jtoolbar, "Center");
        JToolBar jtoolbar1 = new JToolBar();
        jtoolbar1.setBorder(BorderFactory.createEmptyBorder());
        jtoolbar1.setLayout(new FlowLayout(3));
        jtoolbar1.setFloatable(false);
        jtoolbar1.setFont(new Font("SansSerif", 0, 8));
        jpanel.add(jtoolbar1, "South");
        createButtons(jtoolbar);
        jtoolbar.add(Box.createHorizontalGlue());
        jtoolbar.add(Box.createHorizontalGlue());
        jtoolbar.add(Box.createHorizontalGlue());
        jtoolbar.add(Box.createHorizontalGlue());
        createPageNavigation(jtoolbar);
        createPageSizingButtons(jtoolbar1);
        setupScreen();
    }

    protected void setupScreen()
    {
        Dimension dimension = Toolkit.getDefaultToolkit().getScreenSize();
        int i = dimension.width / 2;
        int j = dimension.height / 2;
        if(i < minimumScreenWidth)
            i = minimumScreenWidth;
        setSize(i, j);
        if(showThumbnails)
            displayPane.setDividerLocation(thumbLocation);
        setLocationRelativeTo(null);
        setDefaultCloseOperation(0);
        addWindowListener(new FrameCloser());
        show();
    }

    protected void createPageSizingButtons(JToolBar jtoolbar)
    {
        JLabel jlabel = new JLabel(getMessage("PdfViewerTooltip.scaling"));
        jtoolbar.add(jlabel);
        jlabel.setToolTipText(getMessage("PdfViewerTooltip.zoomin"));
        jtoolbar.add(scalingBox);
        scalingBox.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent actionevent)
            {
                if(!isProcessing && selectedFile != null)
                    zoom();
            }

        });
        jtoolbar.add(Box.createHorizontalGlue());
        JLabel jlabel1 = new JLabel(getMessage("PdfViewerRotation.text"));
        jlabel1.setToolTipText(getMessage("PdfViewerTooltip.rotation"));
        jtoolbar.add(jlabel1);
        jtoolbar.add(rotationBox);
        rotationBox.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent actionevent)
            {
                if(selectedFile != null)
                {
                    rotation = Integer.parseInt((String)rotationBox.getSelectedItem());
                    zoom();
                    decode_pdf.updateUI();
                }
            }

        });
        statusBar.setColorForSubroutines(Color.blue);
        jtoolbar.add(statusBar.getStatusObject());
        JToolBar jtoolbar1 = new JToolBar();
        jtoolbar1.setBorder(BorderFactory.createEmptyBorder());
        jtoolbar1.setLayout(new FlowLayout(3));
        jtoolbar1.setFloatable(false);
        jtoolbar1.setFont(new Font("SansSerif", 2, 10));
        jtoolbar1.add(new JLabel(getMessage("PdfViewerCursorLoc")));
        jtoolbar.add(Box.createHorizontalGlue());
        jtoolbar1.add(coords);
        coords.setBackground(Color.white);
        coords.setOpaque(true);
        coords.setBorder(BorderFactory.createLineBorder(Color.black, 1));
        jtoolbar.add(jtoolbar1);
    }

    protected void createPressedLook(AbstractButton abstractbutton, ImageIcon imageicon)
    {
        BufferedImage bufferedimage = new BufferedImage(imageicon.getIconWidth() + 2, imageicon.getIconHeight() + 2, 2);
        Graphics2D graphics2d = (Graphics2D)bufferedimage.getGraphics();
        graphics2d.drawImage(imageicon.getImage(), 1, 1, null);
        graphics2d.dispose();
        ImageIcon imageicon1 = new ImageIcon(bufferedimage);
        abstractbutton.setPressedIcon(imageicon1);
    }

    private void createButtons(JToolBar jtoolbar)
    {
        openButton(jtoolbar);
        JButton jbutton = new JButton();
        jbutton.setBorderPainted(false);
        ImageIcon imageicon = new ImageIcon(getClass().getResource("/org/jpedal/examples/simpleviewer/print.gif"));
        jbutton.setIcon(imageicon);
        createPressedLook(jbutton, imageicon);
        jbutton.setToolTipText(getMessage("PdfViewerPrint.tip"));
        jtoolbar.add(jbutton);
        jbutton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent actionevent)
            {
                if(SimpleViewer.printingThreads > 0)
                    JOptionPane.showMessageDialog(c, getMessage("PdfViewerPrintWait.message"));
                else
                if(isProcessing)
                    JOptionPane.showMessageDialog(c, getMessage("PdfViewerDecodeWait.message"));
                else
                if(selectedFile != null)
                {
                    if(SimpleViewer.printingThreads == 0)
                        printDialog();
                    else
                        JOptionPane.showMessageDialog(c, getMessage("PdfViewerPrintFinish.message"));
                } else
                {
                    JOptionPane.showMessageDialog(c, getMessage("PdfViewerNoFile.message"));
                }
            }

        });
        jtoolbar.add(Box.createHorizontalGlue());
        bookmarksButton.setBorderPainted(false);
        ImageIcon imageicon1 = new ImageIcon(getClass().getResource("/org/jpedal/examples/simpleviewer/bookmarks.gif"));
        bookmarksButton.setIcon(imageicon1);
        createPressedLook(bookmarksButton, imageicon1);
        if(showThumbnails)
            bookmarksButton.setToolTipText(getMessage("PdfViewerShowide.bookmarks"));
        else
            bookmarksButton.setToolTipText(getMessage("PdfViewerShowHide2.bookmarks"));
        jtoolbar.add(bookmarksButton);
        bookmarksButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent actionevent)
            {
                boolean flag = !navOptionsPanel.isVisible();
                navOptionsPanel.setVisible(flag);
                if(flag)
                    displayPane.setDividerLocation(divLocation);
                else
                    displayPane.setDividerLocation(0);
            }

        });
        jtoolbar.add(Box.createHorizontalGlue());
        JButton jbutton1 = new JButton();
        jbutton1.setBorderPainted(false);
        ImageIcon imageicon2 = new ImageIcon(getClass().getResource("/org/jpedal/examples/simpleviewer/find.gif"));
        jbutton1.setIcon(imageicon2);
        createPressedLook(jbutton1, imageicon2);
        jbutton1.setToolTipText(getMessage("PdfSearch.tip"));
        jtoolbar.add(jbutton1);
        jbutton1.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent actionevent)
            {
                if(selectedFile == null)
                    JOptionPane.showMessageDialog(c, "No file open");
                else
                if(!isPDF)
                    JOptionPane.showMessageDialog(c, "This feature can only be used on pdf files");
                else
                if(searchFrame == null || !searchFrame.isVisible())
                {
                    final JPanel nav = new JPanel();
                    nav.setLayout(new BorderLayout());
                    searchFrame = new JFrame();
                    searchFrame.addWindowListener(new WindowListener() {

                        public void windowOpened(WindowEvent windowevent)
                        {
                        }

                        public void windowClosing(WindowEvent windowevent)
                        {
                            listModel.removeAllElements();
                            searchText = null;
                            searchAll = null;
                            nav.removeAll();
                            searchFrame.dispose();
                        }

                        public void windowClosed(WindowEvent windowevent)
                        {
                        }

                        public void windowIconified(WindowEvent windowevent)
                        {
                        }

                        public void windowDeiconified(WindowEvent windowevent)
                        {
                        }

                        public void windowActivated(WindowEvent windowevent)
                        {
                        }

                        public void windowDeactivated(WindowEvent windowevent)
                        {
                        }

                    });
                    nav.add(searchButton, "East");
                    searchText = new JTextField();
                    nav.add(searchText, "Center");
                    searchAll = new JCheckBox();
                    searchAll.setSelected(true);
                    searchAll.setText("Search whole document");
                    nav.add(searchAll, "North");
                    itemFoundCount = 0;
                    textPages.clear();
                    textRectangles.clear();
                    searchCount = new JTextField("Items found " + itemFoundCount);
                    searchCount.setEditable(false);
                    nav.add(searchCount, "South");
                    listModel = new DefaultListModel();
                    results = new SearchList(listModel, textPages);
                    results.setSelectionMode(1);
                    results.addMouseListener(new MouseListener() {

                        public void mouseClicked(MouseEvent mouseevent)
                        {
                            if(!isProcessing)
                            {
                                int j1 = results.getSelectedIndex();
                                decode_pdf.setFoundTextAreas(null);
                                if(j1 != -1)
                                {
                                    Integer integer = new Integer(j1);
                                    Object obj = textPages.get(integer);
                                    if(obj != null)
                                    {
                                        int k1 = ((Integer)obj).intValue();
                                        Rectangle rectangle = (Rectangle)textRectangles.get(integer);
                                        if(currentPage != k1)
                                        {
                                            currentPage = k1;
                                            statusBar.resetStatus("Loading Page " + currentPage);
                                            decode_pdf.setPageParameters(scaling, currentPage);
                                            decodePage(false);
                                        }
                                        int l1 = decode_pdf.getScrollInterval();
                                        int i2 = (int)((float)(rectangle.x - cropX) * scaling) + 25;
                                        int j2 = (int)((float)(cropH - (rectangle.y - cropY)) * scaling) + 25;
                                        int k2 = (int)((float)rectangle.width * scaling);
                                        int l2 = (int)((float)rectangle.height * scaling);
                                        Rectangle rectangle1 = new Rectangle(i2 - l1, j2 - l2 - l1, k2 + l1 * 2, l2 + l1 * 2);
                                        decode_pdf.scrollRectToVisible(rectangle1);
                                        decode_pdf.setFoundTextAreas(rectangle);
                                        decode_pdf.invalidate();
                                        decode_pdf.repaint();
                                    }
                                }
                            }
                        }

                        public void mousePressed(MouseEvent mouseevent)
                        {
                        }

                        public void mouseReleased(MouseEvent mouseevent)
                        {
                        }

                        public void mouseEntered(MouseEvent mouseevent)
                        {
                        }

                        public void mouseExited(MouseEvent mouseevent)
                        {
                        }

                    });
                    if(firstTime)
                    {
                        firstTime = false;
                        searchButton.addActionListener(new ActionListener() {

                            public void actionPerformed(ActionEvent actionevent1)
                            {
                                if(!isSearch)
                                {
                                    try
                                    {
                                        searchText();
                                    }
                                    catch(Exception exception)
                                    {
                                        System.out.println("Exception " + exception);
                                        exception.printStackTrace();
                                    }
                                } else
                                {
                                    searcher.interrupt();
                                    isSearch = false;
                                    searchButton.setText("search");
                                }
                            }

                        });
                    }
                    searchText.addKeyListener(new KeyListener() {

                        public void keyTyped(KeyEvent keyevent)
                        {
                            int j1 = keyevent.getID();
                            if(j1 == 400)
                            {
                                char c2 = keyevent.getKeyChar();
                                if(c2 == '\n')
                                    try
                                    {
                                        searchText();
                                    }
                                    catch(Exception exception)
                                    {
                                        System.out.println("Exception " + exception);
                                        exception.printStackTrace();
                                    }
                            }
                        }

                        public void keyPressed(KeyEvent keyevent)
                        {
                        }

                        public void keyReleased(KeyEvent keyevent)
                        {
                        }

                    });
                    JScrollPane jscrollpane = new JScrollPane();
                    jscrollpane.getViewport().add(results);
                    jscrollpane.setVerticalScrollBarPolicy(20);
                    jscrollpane.setHorizontalScrollBarPolicy(30);
                    jscrollpane.getVerticalScrollBar().setUnitIncrement(80);
                    jscrollpane.getHorizontalScrollBar().setUnitIncrement(80);
                    searchFrame.getContentPane().setLayout(new BorderLayout());
                    searchFrame.getContentPane().add(jscrollpane, "Center");
                    searchFrame.getContentPane().add(nav, "North");
                    char c1 = '\310';
                    int i = SimpleViewer.current.getHeight();
                    int j = SimpleViewer.current.getLocationOnScreen().x;
                    int k = SimpleViewer.current.getWidth() + j;
                    int l = SimpleViewer.current.getLocationOnScreen().y;
                    Dimension dimension = Toolkit.getDefaultToolkit().getScreenSize();
                    int i1 = dimension.width;
                    if(k + c1 > i1)
                    {
                        k = i1 - c1;
                        SimpleViewer.current.setSize(k - j, SimpleViewer.current.getHeight());
                    }
                    searchFrame.setSize(c1, i);
                    searchFrame.setLocation(k, l);
                    searchAll.setFocusable(false);
                    searchText.grabFocus();
                    searchFrame.show();
                } else
                {
                    searchText.grabFocus();
                }
            }


        });
        jtoolbar.add(Box.createHorizontalGlue());
        createPropertyIcon(jtoolbar);
        createFontButton(jtoolbar);
        createAboutButton(jtoolbar);
    }

    protected void openButton(JToolBar jtoolbar)
    {
        JButton jbutton = new JButton();
        ImageIcon imageicon = new ImageIcon(getClass().getResource("/org/jpedal/examples/simpleviewer/open.gif"));
        jbutton.setIcon(imageicon);
        createPressedLook(jbutton, imageicon);
        jbutton.setToolTipText(getMessage("PdfViewerTooltip.open"));
        jbutton.setBorderPainted(false);
        jtoolbar.add(jbutton);
        jbutton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent actionevent)
            {
                if(SimpleViewer.printingThreads > 0)
                    JOptionPane.showMessageDialog(c, getMessage("PdfViewerPrintWait.message"));
                else
                if(isProcessing)
                    JOptionPane.showMessageDialog(c, getMessage("PdfViewerDecodeWait.message"));
                else
                    selectFile();
            }

        });
        jtoolbar.add(Box.createHorizontalGlue());
    }

    protected void createPropertyIcon(JToolBar jtoolbar)
    {
        JButton jbutton = new JButton();
        jbutton.setBorderPainted(false);
        ImageIcon imageicon = new ImageIcon(getClass().getResource("/org/jpedal/examples/simpleviewer/properties.gif"));
        jbutton.setIcon(imageicon);
        createPressedLook(jbutton, imageicon);
        jbutton.setToolTipText(getMessage("PdfViewer.propsMessage"));
        jtoolbar.add(jbutton);
        jbutton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent actionevent)
            {
                showPropertiesBox();
            }

        });
        jtoolbar.add(Box.createHorizontalGlue());
    }

    protected void createAboutButton(JToolBar jtoolbar)
    {
        JButton jbutton = new JButton();
        jbutton.setBorderPainted(false);
        ImageIcon imageicon = new ImageIcon(getClass().getResource("/org/jpedal/examples/simpleviewer/about.gif"));
        jbutton.setIcon(imageicon);
        createPressedLook(jbutton, imageicon);
        jbutton.setToolTipText(getMessage("PdfViewerTooltip.about"));
        jtoolbar.add(jbutton);
        jbutton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent actionevent)
            {
                showInfoBox();
            }

        });
    }

    protected void createFontButton(JToolBar jtoolbar)
    {
        JButton jbutton = new JButton();
        jbutton.setBorderPainted(false);
        ImageIcon imageicon = new ImageIcon(getClass().getResource("/org/jpedal/examples/simpleviewer/font.gif"));
        jbutton.setIcon(imageicon);
        createPressedLook(jbutton, imageicon);
        jbutton.setToolTipText(getMessage("PdfViewerFontDetails.text"));
        jtoolbar.add(jbutton);
        jbutton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent actionevent)
            {
                showFontsBox();
            }

        });
        jtoolbar.add(Box.createHorizontalGlue());
    }

    protected void createPageNavigation(JToolBar jtoolbar)
    {
        java.net.URL url = getClass().getResource("/org/jpedal/examples/simpleviewer/start.gif");
        first.setBorderPainted(false);
        first.setIcon(new ImageIcon(url));
        createPressedLook(first, new ImageIcon(url));
        first.setToolTipText(getMessage("PdfViewerRewindToStart"));
        jtoolbar.add(first);
        first.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent actionevent)
            {
                if(selectedFile != null && pageCount > 1 && currentPage != 1)
                    back(currentPage - 1);
            }

        });
        java.net.URL url1 = getClass().getResource("/org/jpedal/examples/simpleviewer/fback.gif");
        fback.setBorderPainted(false);
        fback.setIcon(new ImageIcon(url1));
        createPressedLook(fback, new ImageIcon(url1));
        fback.setToolTipText(getMessage("PdfViewerRewind10"));
        jtoolbar.add(fback);
        fback.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent actionevent)
            {
                if(selectedFile != null)
                    back(10);
            }

        });
        java.net.URL url2 = getClass().getResource("/org/jpedal/examples/simpleviewer/back.gif");
        back.setBorderPainted(false);
        back.setIcon(new ImageIcon(url2));
        createPressedLook(back, new ImageIcon(url2));
        back.setToolTipText(getMessage("PdfViewerRewind1"));
        jtoolbar.add(back);
        back.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent actionevent)
            {
                if(selectedFile != null)
                    back(1);
            }

        });
        jtoolbar.add(pageCounter1);
        jtoolbar.add(pageCounter2);
        jtoolbar.add(pageCounter3);
        java.net.URL url3 = getClass().getResource("/org/jpedal/examples/simpleviewer/forward.gif");
        forward.setBorderPainted(false);
        forward.setIcon(new ImageIcon(url3));
        createPressedLook(forward, new ImageIcon(url3));
        forward.setToolTipText(getMessage("PdfViewerForward1"));
        jtoolbar.add(forward);
        forward.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent actionevent)
            {
                if(selectedFile != null)
                    forward(1);
            }

        });
        java.net.URL url4 = getClass().getResource("/org/jpedal/examples/simpleviewer/fforward.gif");
        fforward.setBorderPainted(false);
        fforward.setIcon(new ImageIcon(url4));
        createPressedLook(fforward, new ImageIcon(url4));
        fforward.setToolTipText(getMessage("PdfViewerForward10"));
        jtoolbar.add(fforward);
        fforward.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent actionevent)
            {
                if(selectedFile != null)
                    forward(10);
            }

        });
        java.net.URL url5 = getClass().getResource("/org/jpedal/examples/simpleviewer/end.gif");
        end.setBorderPainted(false);
        end.setIcon(new ImageIcon(url5));
        createPressedLook(end, new ImageIcon(url5));
        end.setToolTipText(getMessage("PdfViewerForwardLast"));
        jtoolbar.add(end);
        end.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent actionevent)
            {
                if(selectedFile != null && pageCount > 1 && pageCount - currentPage > 0)
                    forward(pageCount - currentPage);
            }

        });
        jtoolbar.add(Box.createHorizontalGlue());
    }

    protected void initGUIComponents()
    {
        String as[] = {
            getMessage("PdfViewerScaleWindow.text"), getMessage("PdfViewerScaleHeight.text"), getMessage("PdfViewerScaleWidth.text"), "25", "50", "75", "100", "125", "150", "200", 
            "250", "500", "750", "1000"
        };
        scalingValues = as;
        float af[] = {
            1.0F, 1.0F, 1.0F, 0.25F, 0.5F, 0.75F, 1.0F, 1.25F, 1.5F, 2.0F, 
            2.5F, 5F, 7.5F, 10F
        };
        scalingFloatValues = af;
        scalingBox = new JComboBox(as);
        if(!showThumbnails)
            navOptionsPanel.setVisible(false);
        pageCounter1 = new JLabel(getMessage("PdfViewerPageLabel.text"));
        pageCounter1.setOpaque(false);
        pageCounter2.setEditable(true);
        pageCounter2.setToolTipText(getMessage("PdfViewerTooltip.goto"));
        pageCounter2.setBorder(BorderFactory.createLineBorder(Color.black));
        pageCounter2.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent actionevent)
            {
                String s = pageCounter2.getText().trim();
                int i;
                try
                {
                    i = Integer.parseInt(s);
                    if((i > decode_pdf.getPageCount()) | (i < 1))
                    {
                        JOptionPane.showMessageDialog(c, getMessage("PdfViewerPageLabel.text") + " " + s + " " + getMessage("PdfViewerOutOfRange.text") + " " + decode_pdf.getPageCount());
                        i = currentPage;
                        pageCounter2.setText("" + currentPage);
                    }
                }
                catch(Exception exception)
                {
                    JOptionPane.showMessageDialog(c, ">" + s + "< " + getMessage("PdfViewerInvalidNumber.text"));
                    i = currentPage;
                    pageCounter2.setText("" + currentPage);
                }
                if(!isProcessing && currentPage != i)
                {
                    currentPage = i;
                    decodePage(false);
                    zoom();
                }
            }

        });
        pageCounter3 = new JLabel(getMessage("PdfViewerOfLabel.text") + " ");
        pageCounter3.setOpaque(false);
        scalingBox.setBackground(Color.white);
        scalingBox.setEditable(true);
        scalingBox.setPreferredSize(new Dimension(85, 25));
        scalingBox.setSelectedIndex(0);
        rotationBox.setBackground(Color.white);
        rotationBox.setSelectedIndex(0);
    }

    protected void createMenu(JMenuBar jmenubar, boolean flag)
    {
        JMenu jmenu = new JMenu(getMessage("PdfViewerFile.text"));
        jmenubar.add(jmenu);
        if(flag)
        {
            JMenu jmenu1 = new JMenu(getMessage("PdfViewerHelp.text"));
            jmenubar.add(jmenu1);
            infoOption(jmenu1);
        }
        JMenu jmenu2 = new JMenu(getMessage("PdfViewerMenu.options"));
        jmenubar.add(jmenu2);
        optionAutoscroll(jmenu2);
        if(flag)
        {
            openOption(jmenu);
            saveOption(jmenu);
        }
        fontsOption(jmenu);
        docInfoOption(jmenu);
        pageInfoOption(jmenu);
        if(flag)
            printOption(jmenu);
        exitOption(jmenu);
    }

    private void optionAutoscroll(JMenu jmenu)
    {
        JMenuItem jmenuitem = new JMenuItem(getMessage("PdfViewerTitle.autoscroll"));
        jmenuitem.setToolTipText(getMessage("PdfViewerTooltip.autoscroll"));
        jmenuitem.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent actionevent)
            {
                allowScrolling = !allowScrolling;
            }

        });
        jmenu.add(jmenuitem);
    }

    private void exitOption(JMenu jmenu)
    {
        JMenuItem jmenuitem = new JMenuItem(getMessage("PdfViewerExit.text"));
        jmenuitem.setToolTipText(getMessage("PdfViewerShutdown.text"));
        jmenuitem.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent actionevent)
            {
                if(SimpleViewer.printingThreads > 0)
                    JOptionPane.showMessageDialog(c, getMessage("PdfViewerStillPrinting.text"));
                else
                    exit();
            }

        });
        jmenu.add(jmenuitem);
    }

    private void printOption(JMenu jmenu)
    {
        JMenuItem jmenuitem = new JMenuItem(getMessage("PdfViewerPrint.text"));
        jmenu.add(jmenuitem);
        jmenuitem.setToolTipText(getMessage("PdfViewerTooltip.print"));
        jmenuitem.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent actionevent)
            {
                if(selectedFile != null)
                {
                    if(SimpleViewer.printingThreads == 0)
                        printDialog();
                    else
                        JOptionPane.showMessageDialog(c, getMessage("PdfViewerPrintFinish.message"));
                } else
                {
                    JOptionPane.showMessageDialog(c, getMessage("PdfViewerNoFile.message"));
                }
            }

        });
    }

    private void pageInfoOption(JMenu jmenu)
    {
        JMenuItem jmenuitem = new JMenuItem(getMessage("PdfViewerMenu.pageSize"));
        jmenuitem.setToolTipText(getMessage("PdfViewerTooltip.pageSize"));
        jmenuitem.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent actionevent)
            {
                PdfPageData pdfpagedata = decode_pdf.getPdfPageData();
                if(pdfpagedata != null)
                {
                    JPanel jpanel = new JPanel();
                    jpanel.setLayout(new BoxLayout(jpanel, 3));
                    JLabel jlabel = new JLabel(getMessage("PdfViewerCoords.text"));
                    jlabel.setFont(headFont);
                    jpanel.add(jlabel);
                    jpanel.add(Box.createRigidArea(new Dimension(0, 5)));
                    JLabel jlabel1 = new JLabel(getMessage("PdfViewermediaBox.text") + pdfpagedata.getMediaValue(currentPage));
                    jlabel1.setFont(textFont);
                    jpanel.add(jlabel1);
                    jpanel.add(Box.createRigidArea(new Dimension(0, 5)));
                    JLabel jlabel2 = new JLabel(getMessage("PdfViewercropBox.text") + pdfpagedata.getCropValue(currentPage));
                    jlabel2.setFont(textFont);
                    jpanel.add(jlabel2);
                    jpanel.add(Box.createRigidArea(new Dimension(0, 5)));
                    JLabel jlabel3 = new JLabel(getMessage("PdfViewerrotation.text") + pdfpagedata.getRotation(currentPage));
                    jlabel3.setFont(textFont);
                    jpanel.add(jlabel3);
                    jpanel.add(Box.createRigidArea(new Dimension(0, 5)));
                    JOptionPane.showMessageDialog(c, jpanel, getMessage("PdfViewersize.text"), -1);
                } else
                {
                    JOptionPane.showMessageDialog(c, getMessage("PdfVieweremptyFile.message"), getMessage("PdfViewerTooltip.pageSize"), -1);
                }
            }

        });
        jmenu.add(jmenuitem);
    }

    private void docInfoOption(JMenu jmenu)
    {
        JMenuItem jmenuitem = new JMenuItem(getMessage("PdfViewerProperties"));
        jmenuitem.setToolTipText(getMessage("PdfViewerMenu.props"));
        jmenuitem.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent actionevent)
            {
                showPropertiesBox();
            }

        });
        jmenu.add(jmenuitem);
    }

    private void fontsOption(JMenu jmenu)
    {
        JMenuItem jmenuitem = new JMenuItem(getMessage("PdfViewerMenu.fonts"));
        jmenuitem.setToolTipText(getMessage("PdfViewerTooltip.fonts"));
        jmenuitem.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent actionevent)
            {
                showFontsBox();
            }

        });
        jmenu.add(jmenuitem);
    }

    private void infoOption(JMenu jmenu)
    {
        JMenuItem jmenuitem = new JMenuItem(getMessage("PdfViewerMenu.about"));
        jmenuitem.setToolTipText(getMessage("PdfViewerTooltip.about"));
        jmenuitem.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent actionevent)
            {
                showInfoBox();
            }

        });
        jmenu.add(jmenuitem);
    }

    private void openOption(JMenu jmenu)
    {
        JMenuItem jmenuitem = new JMenuItem(getMessage("PdfViewerOpen.text"));
        jmenuitem.setToolTipText(getMessage("PdfViewerTooltip.open"));
        jmenuitem.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent actionevent)
            {
                if(SimpleViewer.printingThreads > 0)
                    JOptionPane.showMessageDialog(c, getMessage("PdfViewerPrintWait.message"));
                else
                if(isProcessing)
                    JOptionPane.showMessageDialog(c, getMessage("PdfViewerDecodeWait.message"));
                else
                    selectFile();
            }

        });
        jmenu.add(jmenuitem);
    }

    private void saveOption(JMenu jmenu)
    {
        JMenuItem jmenuitem = new JMenuItem(getMessage("PdfViewerSave.text"));
        jmenuitem.setToolTipText(getMessage("PdfViewerTooltip.save"));
        jmenuitem.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent actionevent)
            {
                JFileChooser jfilechooser = new JFileChooser(inputDir);
                jfilechooser.setSelectedFile(new File(inputDir + "/" + selectedFile));
                jfilechooser.addChoosableFileFilter(new FileFilterer(new String[] {
                    "pdf"
                }, "Pdf (*.pdf)"));
                jfilechooser.setFileSelectionMode(0);
                int i = jfilechooser.showSaveDialog(null);
                if(i == 0)
                {
                    File file = jfilechooser.getSelectedFile();
                    FileInputStream fileinputstream = null;
                    FileOutputStream fileoutputstream = null;
                    try
                    {
                        fileinputstream = new FileInputStream(selectedFile);
                        fileoutputstream = new FileOutputStream(file.toString() + ".pdf");
                        byte abyte0[] = new byte[4096];
                        int j;
                        while((j = fileinputstream.read(abyte0)) != -1) 
                            fileoutputstream.write(abyte0, 0, j);
                    }
                    catch(Exception exception)
                    {
                        exception.printStackTrace();
                    }
                    try
                    {
                        fileinputstream.close();
                        fileoutputstream.close();
                    }
                    catch(Exception exception1)
                    {
                        exception1.printStackTrace();
                    }
                }
            }

        });
        jmenu.add(jmenuitem);
    }

    protected void showPropertiesBox()
    {
        PdfFileInformation pdffileinformation = decode_pdf.getFileInformationData();
        if(pdffileinformation != null)
        {
            JPanel jpanel = new JPanel();
            jpanel.setOpaque(false);
            jpanel.setLayout(new BoxLayout(jpanel, 1));
            JLabel jlabel = new JLabel(getMessage("PdfViewerGeneral"));
            jlabel.setFont(headFont);
            jlabel.setOpaque(false);
            jpanel.add(jlabel);
            jlabel.setAlignmentX(0.5F);
            JLabel jlabel1 = new JLabel(getMessage("PdfViewerFileName") + selectedFile);
            jlabel1.setFont(textFont);
            jlabel1.setOpaque(false);
            jpanel.add(jlabel1);
            JLabel jlabel2 = new JLabel(getMessage("PdfViewerFilePath") + inputDir);
            jlabel2.setFont(textFont);
            jlabel2.setOpaque(false);
            jpanel.add(jlabel2);
            JLabel jlabel3 = new JLabel(getMessage("PdfViewerFileSize") + size + " K");
            jlabel3.setFont(textFont);
            jlabel2.setOpaque(false);
            jpanel.add(jlabel3);
            JLabel jlabel4 = new JLabel(getMessage("PdfViewerPageCount") + pageCount);
            jlabel4.setOpaque(false);
            jlabel4.setFont(textFont);
            jpanel.add(jlabel4);
            JLabel jlabel5 = new JLabel("PDF " + decode_pdf.getPDFVersion());
            jlabel5.setOpaque(false);
            jlabel5.setFont(textFont);
            jpanel.add(jlabel5);
            jpanel.add(Box.createRigidArea(new Dimension(0, 15)));
            JLabel jlabel6 = new JLabel(getMessage("PdfViewerProperties"));
            jlabel6.setFont(headFont);
            jlabel6.setOpaque(false);
            jpanel.add(jlabel6);
            jlabel6.setAlignmentX(0.5F);
            String as[] = pdffileinformation.getFieldValues();
            String as1[] = pdffileinformation.getFieldNames();
            int i = as1.length;
            JLabel ajlabel[] = new JLabel[i];
            for(int j = 0; j < i; j++)
                if(as[j].length() > 0)
                {
                    ajlabel[j] = new JLabel(as1[j] + " = " + as[j]);
                    ajlabel[j].setFont(textFont);
                    ajlabel[j].setOpaque(false);
                    jpanel.add(ajlabel[j]);
                }

            jpanel.add(Box.createRigidArea(new Dimension(0, 5)));
            String s = pdffileinformation.getFileXMLMetaData();
            if(s.length() > 0)
            {
                JLabel jlabel7 = new JLabel("XML metadata");
                jlabel7.setFont(headFont);
                jpanel.add(jlabel7);
                JTextArea jtextarea = new JTextArea();
                jtextarea.setRows(5);
                jtextarea.setColumns(15);
                jtextarea.setLineWrap(true);
                jtextarea.setText(s);
                jpanel.add(new JScrollPane(jtextarea));
                jtextarea.setCaretPosition(0);
                jtextarea.setOpaque(false);
                jpanel.add(Box.createRigidArea(new Dimension(0, 5)));
            }
            jpanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
            JOptionPane.showMessageDialog(this, jpanel, getMessage("PdfViewerMenu.props"), -1);
        } else
        {
            JOptionPane.showMessageDialog(this, getMessage("PdfVieweremptyFile.message"), getMessage("PdfViewerMenu.props"), -1);
        }
    }

    private void showFontsBox()
    {
        JPanel jpanel = new JPanel();
        jpanel.setPreferredSize(new Dimension(400, 300));
        jpanel.setOpaque(false);
        jpanel.setEnabled(false);
        jpanel.setLayout(new BoxLayout(jpanel, 3));
        JLabel jlabel = new JLabel(getMessage("PdfViewerFontList.title"));
        jlabel.setFont(headFont);
        jlabel.setOpaque(false);
        jpanel.add(jlabel);
        jlabel.setAlignmentX(0.5F);
        jpanel.add(Box.createRigidArea(new Dimension(0, 5)));
        String s = decode_pdf.getFontsInFile();
        if(s.length() > 0)
        {
            JTextArea jtextarea = new JTextArea();
            jtextarea.setLineWrap(false);
            jtextarea.setText(s);
            jpanel.add(jtextarea);
            jtextarea.setCaretPosition(0);
            jtextarea.setOpaque(false);
            jpanel.add(Box.createRigidArea(new Dimension(0, 5)));
        }
        JOptionPane.showMessageDialog(this, jpanel, getMessage("PdfViewerFontList.subtitle"), -1);
    }

    protected void showInfoBox()
    {
        JPanel jpanel = new JPanel();
        jpanel.setPreferredSize(new Dimension(400, 230));
        jpanel.setOpaque(false);
        jpanel.setLayout(new BoxLayout(jpanel, 1));
        JLabel jlabel = new JLabel(getMessage("PdfViewerInfo.title"));
        jlabel.setOpaque(false);
        jlabel.setFont(headFont);
        jlabel.setAlignmentX(0.5F);
        jpanel.add(jlabel);
        jpanel.add(Box.createRigidArea(new Dimension(0, 15)));
        String s = getMessage("PdfViewerInfo1") + getMessage("PdfViewerInfo2");
        if(s.length() > 0)
        {
            JTextArea jtextarea = new JTextArea();
            jtextarea.setOpaque(false);
            jtextarea.setText(s);
            jtextarea.setLineWrap(true);
            jtextarea.setWrapStyleWord(true);
            jpanel.add(jtextarea);
            jtextarea.setAlignmentX(0.5F);
        }
        ImageIcon imageicon = new ImageIcon(getClass().getResource("/org/jpedal/examples/simpleviewer/logo.gif"));
        jpanel.add(Box.createRigidArea(new Dimension(0, 25)));
        JLabel jlabel1 = new JLabel(imageicon);
        jlabel1.setAlignmentX(0.5F);
        jpanel.add(jlabel1);
        JLabel jlabel2 = new JLabel("http://www.jpedal.org");
        jlabel2.setForeground(Color.blue);
        jlabel2.setAlignmentX(0.5F);
        jpanel.add(jlabel2);
        jpanel.add(Box.createRigidArea(new Dimension(0, 5)));
        jpanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
        JOptionPane.showMessageDialog(this, jpanel, getMessage("PdfViewerInfo3"), -1);
    }

    protected void printDialog()
    {
        if(!isPDF)
        {
            JOptionPane.showMessageDialog(c, "Printing is not available on images at this time");
        } else
        {
            printingThreads++;
            Thread thread = new Thread() {

                public void run()
                {
                    Object obj = null;
                    boolean flag = false;
                    try
                    {
                        String s = getMessage("PdfViewera4Border");
                        decode_pdf.enableScaledPrinting(false);
                        PrinterJob printerjob = PrinterJob.getPrinterJob();
                        PageFormat pageformat = printerjob.defaultPage();
                        Paper paper = new Paper();
                        paper.setSize(595D, 842D);
                        paper.setImageableArea(43D, 43D, 509D, 756D);
                        boolean flag1 = true;
                        do
                        {
                            if(!flag1)
                                break;
                            String as[] = {
                                getMessage("PdfViewerHelp.text"), getMessage("PdfViewera4"), getMessage("PdfViewera4borderless"), getMessage("PdfViewera5"), getMessage("PdfVieweruseScaleOnPrinting.text"), getMessage("PdfViewerCancel.text"), getMessage("PdfViewerPrint.text")
                            };
                            JTextArea jtextarea = new JTextArea(getMessage("PdfViewerPrintInfo1") + "\n=============\n" + getMessage("PdfViewerPrintInfo2") + " " + s + "\n" + getMessage("PdfViewerPrintInfo3") + paper.getWidth() + "\n" + getMessage("PdfViewerPrintInfo4") + paper.getHeight() + "\n" + getMessage("PdfViewerPrintInfo5") + paper.getImageableX() + " " + paper.getImageableY() + " " + paper.getImageableWidth() + " " + paper.getImageableHeight() + "\n" + getMessage("PdfViewerPrintInfo6") + "\n\n" + getMessage("PdfViewerPrintInfo7") + "\n===========\n" + getMessage("PdfViewerPrintInfo8") + "\n" + getMessage("PdfViewerPrintInfo9") + "\n" + getMessage("PdfViewerPrintInfo10") + "\n\n" + getMessage("PdfViewerPrintScalingValue") + decode_pdf.getScaleForPrinting());
                            jtextarea.setColumns(30);
                            jtextarea.setWrapStyleWord(true);
                            int i = JOptionPane.showOptionDialog(c, jtextarea, getMessage("PdfViewerPrintOptions.title"), 0, 1, null, as, as[0]);
                            switch(i)
                            {
                            case 0: // '\0'
                                JTextArea jtextarea1 = new JTextArea(getMessage("PdfViewerHelp.message"));
                                JOptionPane.showMessageDialog(c, jtextarea1);
                                break;

                            case 1: // '\001'
                                s = getMessage("PdfViewera4");
                                paper.setSize(595D, 842D);
                                paper.setImageableArea(43D, 43D, 509D, 756D);
                                break;

                            case 2: // '\002'
                                s = getMessage("PdfViewera4borderless");
                                paper.setSize(595D, 842D);
                                paper.setImageableArea(0.0D, 0.0D, 595D, 842D);
                                JOptionPane.showMessageDialog(c, getMessage("PdfViewerPrintWarning"));
                                break;

                            case 3: // '\003'
                                s = getMessage("PdfViewera5");
                                paper.setSize(297D, 421D);
                                paper.setImageableArea(23D, 23D, 254D, 378D);
                                break;

                            case 4: // '\004'
                                if(decode_pdf.usePageScaling)
                                    decode_pdf.enableScaledPrinting(false);
                                else
                                    decode_pdf.enableScaledPrinting(true);
                                break;

                            case 5: // '\005'
                                flag = true;
                                flag1 = false;
                                break;

                            case 6: // '\006'
                                flag1 = false;
                                break;

                            default:
                                System.out.println("No selection");
                                break;
                            }
                        } while(true);
                        if(!flag)
                        {
                            pageformat.setPaper(paper);
                            printerjob.setPageable(decode_pdf);
                            decode_pdf.setPageFormat(pageformat);
                            boolean flag2 = printerjob.printDialog();
                            if(flag2)
                                printerjob.print();
                        }
                    }
                    catch(PrinterException printerexception)
                    {
                        LogWriter.writeLog("Exception " + printerexception + " printing");
                        JOptionPane.showMessageDialog(c, printerexception.getMessage() + " " + printerexception + " " + " " + printerexception.getCause());
                    }
                    catch(Exception exception)
                    {
                        LogWriter.writeLog("Exception " + exception + " printing");
                        exception.printStackTrace();
                        JOptionPane.showMessageDialog(c, "Exception " + exception);
                    }
                    catch(Error error)
                    {
                        LogWriter.writeLog("Error " + error + " printing");
                        JOptionPane.showMessageDialog(c, "Error " + error);
                    }
                    if(!flag && !decode_pdf.isPageSuccessful())
                        JOptionPane.showMessageDialog(c, "Problems encountered\n " + decode_pdf.getPageFailureMessage() + "\n");
                    SimpleViewer.printingThreads--;
                    decode_pdf.invalidate();
                    decode_pdf.updateUI();
                    repaint();
                    if(!flag)
                        JOptionPane.showMessageDialog(c, getMessage("PdfViewerPrintingFinished"));
                }

            };
            thread.start();
        }
    }

    public static void main(String args[])
    {
        current = new SimpleViewer();
        if(args.length > 0)
            current.setupViewer(args[0]);
        else
            current.setupViewer();
    }

    public void selectFile()
    {
        JFileChooser jfilechooser = new JFileChooser(inputDir);
        if(selectedFile != null)
            jfilechooser.setSelectedFile(new File(selectedFile));
        jfilechooser.setFileSelectionMode(0);
        String as[] = {
            "pdf"
        };
        String as1[] = {
            "png", "tif", "tiff", "jpg", "jpeg"
        };
        jfilechooser.addChoosableFileFilter(new FileFilterer(as1, "Images (Tiff, Jpeg,Png)"));
        jfilechooser.addChoosableFileFilter(new FileFilterer(as, "Pdf (*.pdf)"));
        int i = jfilechooser.showOpenDialog(c);
        decode_pdf.invalidate();
        decode_pdf.repaint();
        File file = jfilechooser.getSelectedFile();
        if(file != null && i == 0)
        {
            try
            {
                selectedFile = file.getCanonicalPath();
                size = file.length() >> 10;
                inputDir = jfilechooser.getCurrentDirectory().getCanonicalPath();
                setViewerTitle(null);
            }
            catch(Exception exception)
            {
                System.err.println("Exception " + exception + " getting paths");
            }
            File file1 = new File(selectedFile);
            if(!file1.exists())
                JOptionPane.showMessageDialog(this, getMessage("PdfViewerFile.text") + selectedFile + getMessage("PdfViewerNotExist"));
            else
            if((selectedFile != null) & (!isProcessing))
            {
                decode_pdf.repaint();
                if(isProcessing | (worker != null) && thumbnails.drawing)
                {
                    thumbnails.interrupt = true;
                    while(thumbnails.drawing) 
                        try
                        {
                            Thread.sleep(20L);
                        }
                        catch(InterruptedException interruptedexception)
                        {
                            interruptedexception.printStackTrace();
                        }
                    thumbnails.interrupt = false;
                }
                decode_pdf.flushObjectValues(true);
                decode_pdf.resetViewableArea();
                thumbnails.removeComponentListener(painter);
                openFile(selectedFile);
            }
        } else
        {
            decode_pdf.repaint();
            JOptionPane.showMessageDialog(this, getMessage("PdfViewerNoSelection"));
        }
    }

    public void setViewerTitle(String s)
    {
        if(s != null)
            setTitle(s);
        else
        if(titleMessage == null)
            setTitle(getMessage("PdfViewerCom.message") + "FRF-2.60b10" + " ) " + selectedFile);
        else
            setTitle(titleMessage + selectedFile);
    }

    public void openFile(File file, String s)
    {
        showThumbnails = showThumbnailsdefault;
        maxViewY = 0;
        try
        {
            boolean flag = openUpFile(file.getCanonicalPath());
            Node node = decode_pdf.getOutlineAsXML().getFirstChild();
            DefaultMutableTreeNode defaultmutabletreenode = new DefaultMutableTreeNode("Root");
            if(node != null)
                readChildNodes(node, defaultmutabletreenode);
            Object obj = pageLookupTable.get(s);
            if(obj == null)
                throw new PdfException("Unknown bookmark " + s);
            int i = Integer.parseInt((String)obj);
            currentPage = i;
            if(flag)
                processPage();
        }
        catch(Exception exception)
        {
            System.err.println("Exception " + exception + " processing file");
            isProcessing = false;
        }
    }

    public void openFile(File file, int i)
    {
        showThumbnails = showThumbnailsdefault;
        maxViewY = 0;
        try
        {
            boolean flag = openUpFile(file.getCanonicalPath());
            currentPage = i;
            if(flag)
                processPage();
        }
        catch(Exception exception)
        {
            System.err.println("Exception " + exception + " processing file");
            isProcessing = false;
        }
    }

    private void openFile(String s)
    {
        showThumbnails = showThumbnailsdefault;
        pagesDecoded.clear();
        if(searchFrame != null)
        {
            searchFrame.hide();
            searchFrame.dispose();
        }
        maxViewY = 0;
        isPDF = s.toLowerCase().endsWith(".pdf");
        boolean flag = openUpFile(s);
        currentPage = 1;
        try
        {
            if(flag)
                processPage();
        }
        catch(Exception exception)
        {
            System.err.println("Exception " + exception + " decoding file");
            exception.printStackTrace();
        }
        isProcessing = false;
    }

    private boolean openUpFile(String s)
    {
        maxViewY = 0;
        boolean flag = true;
        scaling = 1.0F;
        scalingBox.setSelectedIndex(0);
        decode_pdf.closePdfFile();
        try
        {
            if(isPDF)
            {
                decode_pdf.openPdfFile(s);
            } else
            {
                boolean flag1 = s.toLowerCase().indexOf(".tif") != -1;
                if(flag1)
                    try
                    {
                        img = JAI.create("fileload", s).getAsBufferedImage();
                    }
                    catch(Exception exception1)
                    {
                        LogWriter.writeLog("Exception " + exception1 + " loading " + s);
                    }
                else
                    try
                    {
                        img = ImageIO.read(new File(s));
                    }
                    catch(Exception exception2)
                    {
                        LogWriter.writeLog("Exception " + exception2 + " loading " + s);
                    }
                decode_pdf.resetForNonPDFPage();
                PdfPageData pdfpagedata = decode_pdf.getPdfPageData();
                pdfpagedata.setMediaBox("0 0 " + img.getWidth() + " " + img.getHeight());
                pdfpagedata.checkSizeSet(1);
                resetRotationBox();
            }
            statusBar.resetStatus("opening file");
            if(decode_pdf.isEncrypted() && !decode_pdf.isFileViewable())
            {
                flag = false;
                String s1 = JOptionPane.showInputDialog(this, getMessage("PdfViewerPassword.message"));
                if(s1 != null)
                {
                    decode_pdf.setEncryptionPassword(s1);
                    decode_pdf.verifyAccess();
                    if(decode_pdf.isFileViewable())
                        flag = true;
                }
            }
            removeOutlinePanels();
            if(!flag)
            {
                JOptionPane.showMessageDialog(this, getMessage("PdfViewerPasswordRequired.message"));
            } else
            {
                currentFileCount++;
                currentPage = 1;
            }
        }
        catch(Exception exception)
        {
            System.err.println("Exception " + exception + " opening file");
            JOptionPane.showMessageDialog(this, getMessage("PdfViewerOpenerror"));
            System.exit(1);
            isProcessing = false;
        }
        return flag;
    }

    private void processPage()
    {
        if(isPDF && switchModes)
        {
            PdfFileInformation pdffileinformation = decode_pdf.getFileInformationData();
            decode_pdf.setRenderMode(3);
            String as[] = pdffileinformation.getFieldValues();
            String as1[] = pdffileinformation.getFieldNames();
            for(int i = 0; i < as1.length; i++)
            {
                if(as1[i].equals("Creator") | as1[i].equals("Producer"))
                {
                    for(int j = 0; j < ocr.length; j++)
                        if(as[i].equals(ocr[j]))
                        {
                            decode_pdf.setRenderMode(2);
                            decode_pdf.setEnableLegacyJPEGConversion(true);
                        }

                }
                if(!as1[i].equals("Creator"))
                    continue;
                if(as[i].equals("XPP"))
                    decode_pdf.useHiResScreenDisplay(true);
                else
                    decode_pdf.useHiResScreenDisplay(false);
            }

        }
        pageCount = decode_pdf.getPageCount();
        if(!isPDF)
        {
            pageCount = 1;
            decode_pdf.useHiResScreenDisplay(true);
        }
        if(pageCount < currentPage)
        {
            currentPage = pageCount;
            System.err.println(currentPage + " out of range. Opening on last page");
            LogWriter.writeLog(currentPage + " out of range. Opening on last page");
        }
        decode_pdf.setExtractionMode(0, 72, scaling);
        zoom();
        decode_pdf.setPDFBorder(BorderFactory.createLineBorder(Color.black, 1));
        decode_pdf.disableBorderForPrinting();
        pageCounter2.setForeground(Color.black);
        pageCounter2.setText(" " + currentPage);
        pageCounter3.setText(getMessage("PdfViewerOfLabel.text") + " " + pageCount);
        resetRotationBox();
        if(isPDF)
            decodePage(true);
        else
            decodeImage(true);
    }

    private void resetRotationBox()
    {
        PdfPageData pdfpagedata = decode_pdf.getPdfPageData();
        rotation = pdfpagedata.getRotation(currentPage);
        if(rotationBox.getSelectedIndex() != rotation / 90)
            rotationBox.setSelectedIndex(rotation / 90);
        else
            decode_pdf.repaint();
    }

    public void zoom()
    {
        if(decode_pdf != null)
        {
            int i = scalingBox.getSelectedIndex();
            if(i == -1)
            {
                String s = (String)scalingBox.getSelectedItem();
                float f = -1F;
                if(s != null && s.length() > 0)
                {
                    try
                    {
                        f = Float.parseFloat(s);
                    }
                    catch(Exception exception)
                    {
                        f = -1F;
                        int l = s.length();
                        int j1 = 0;
                        do
                        {
                            if(j1 >= l)
                                break;
                            char c1 = s.charAt(j1);
                            if(!((c1 >= '0' && c1 <= '9') | (c1 == '.')))
                                break;
                            j1++;
                        } while(true);
                        if(j1 > 0)
                            s = s.substring(0, j1);
                        if(f == -1F)
                            try
                            {
                                f = Float.parseFloat(s);
                            }
                            catch(Exception exception1)
                            {
                                f = -1F;
                            }
                    }
                    if(f > 1000F)
                        f = 1000F;
                }
                if(f == -1F)
                {
                    i = 0;
                    scalingBox.setSelectedItem(scalingBox.getItemAt(i));
                } else
                {
                    scaling = f / 100F;
                    scalingBox.setSelectedItem(f + "");
                }
            }
            if(i != -1)
                if(i < 3)
                {
                    PdfPageData pdfpagedata = decode_pdf.getPdfPageData();
                    int i1 = pdfpagedata.getRotation(currentPage);
                    int j;
                    int k;
                    if(rotation == 90 || rotation == 270)
                    {
                        j = pdfpagedata.getCropBoxHeight(currentPage);
                        k = pdfpagedata.getCropBoxWidth(currentPage);
                    } else
                    {
                        j = pdfpagedata.getCropBoxWidth(currentPage);
                        k = pdfpagedata.getCropBoxHeight(currentPage);
                    }
                    float f1 = scrollPane.getViewport().getWidth() - 25 - 25 - displayPane.getDividerSize();
                    float f2 = scrollPane.getViewport().getHeight() - 25 - 25;
                    float f3 = 0.0F;
                    float f4 = 0.0F;
                    f3 = f1 / (float)j;
                    f4 = f2 / (float)k;
                    if(i == 0)
                    {
                        if(f3 < f4)
                            scaling = f3;
                        else
                            scaling = f4;
                    } else
                    if(i == 1)
                        scaling = f4;
                    else
                    if(i == 2)
                        scaling = f3;
                } else
                {
                    scaling = scalingFloatValues[i];
                }
            PdfPageData pdfpagedata1 = decode_pdf.getPdfPageData();
            if((float)pdfpagedata1.getCropBoxHeight(currentPage) * scaling < 100F && (float)pdfpagedata1.getCropBoxWidth(currentPage) * scaling < 100F && isPDF)
            {
                scaling = 1.0F;
                scalingBox.setSelectedItem("100");
            }
            if(decode_pdf != null)
                decode_pdf.setPageParameters(scaling, currentPage, rotation);
            decode_pdf.invalidate();
            scrollPane.repaint();
            validate();
        }
    }

    public void forward(int i)
    {
        if(!isProcessing)
        {
            if(currentPage + i <= pageCount)
            {
                currentPage += i;
                statusBar.resetStatus("Loading Page " + currentPage);
                decode_pdf.setPageParameters(scaling, currentPage);
                decodePage(false);
                if(scalingBox.getSelectedIndex() < 3)
                    zoom();
            }
        } else
        {
            JOptionPane.showMessageDialog(this, getMessage("PdfViewerDecodeWait.message"));
        }
    }

    protected void decodePage(final boolean resizePanel)
    {
        decode_pdf.setFoundTextAreas(null);
        currentRectangle = null;
        scalingBox.setEnabled(false);
        rotationBox.setEnabled(false);
        decode_pdf.clearScreen();
        if(isProcessing | (worker != null) && thumbnails.drawing)
        {
            thumbnails.interrupt = true;
            while(thumbnails.drawing) 
                try
                {
                    Thread.sleep(20L);
                }
                catch(InterruptedException interruptedexception)
                {
                    interruptedexception.printStackTrace();
                }
            thumbnails.interrupt = false;
        }
        isProcessing = true;
        if(!hasOutlinesDrawn)
        {
            hasOutlinesDrawn = true;
            createOutlinePanels();
        }
        worker = new SwingWorker() {

            public Object construct()
            {
                try
                {
                    statusBar.updateStatus("Decoding Page", 0);
                    if(resizePanel && showThumbnails)
                        zoom();
                    if(Thread.interrupted())
                        throw new InterruptedException();
                    try
                    {
                        decode_pdf.decodePage(currentPage);
                        PdfPageData pdfpagedata = decode_pdf.getPdfPageData();
                        mediaW = pdfpagedata.getMediaBoxWidth(currentPage);
                        mediaH = pdfpagedata.getMediaBoxHeight(currentPage);
                        mediaX = pdfpagedata.getMediaBoxX(currentPage);
                        mediaY = pdfpagedata.getMediaBoxY(currentPage);
                        cropX = pdfpagedata.getCropBoxX(currentPage);
                        cropY = pdfpagedata.getCropBoxY(currentPage);
                        cropW = pdfpagedata.getCropBoxWidth(currentPage);
                        cropH = pdfpagedata.getCropBoxHeight(currentPage);
                        resetRotationBox();
                        pageAnnotations = decode_pdf.getPdfAnnotsData(null);
                        statusBar.updateStatus("Displaying Page", 0);
                    }
                    catch(Exception exception)
                    {
                        System.err.println("Exception " + exception + " decoding page");
                        exception.printStackTrace();
                    }
                    pageCounter2.setForeground(Color.black);
                    pageCounter2.setText(" " + currentPage);
                    pageCounter3.setText(getMessage("PdfViewerOfLabel.text") + " " + pageCount);
                    if(decode_pdf.hasEmbeddedFonts() && !decode_pdf.supportsEmbeddedFonts())
                        JOptionPane.showMessageDialog(c, "Page contains embedded fonts which may not display correctly using Font substitution.");
                    if(showThumbnails)
                        thumbnails.addNewThumbnails(currentPage, decode_pdf);
                    else
                        isProcessing = false;
                    setViewerTitle(null);
                    if(showThumbnails)
                    {
                        thumbnails.setupThumbnailsOnDecode(currentPage, decode_pdf);
                        isProcessing = false;
                        thumbnails.createThumbnailsOnDecode(currentPage, decode_pdf);
                    }
                }
                catch(Exception exception1)
                {
                    setViewerTitle(null);
                }
                selectBookmark();
                statusBar.setProgress(100);
                scalingBox.setEnabled(true);
                rotationBox.setEnabled(true);
                addExampleListeners();
                return null;
            }

        };
        worker.start();
    }

    protected void decodeImage(final boolean resizePanel)
    {
        decode_pdf.setFoundTextAreas(null);
        currentRectangle = null;
        scalingBox.setEnabled(false);
        rotationBox.setEnabled(false);
        decode_pdf.clearScreen();
        if(isProcessing | (worker != null) && thumbnails.drawing)
        {
            thumbnails.interrupt = true;
            while(thumbnails.drawing) 
                try
                {
                    Thread.sleep(20L);
                }
                catch(InterruptedException interruptedexception)
                {
                    interruptedexception.printStackTrace();
                }
            thumbnails.interrupt = false;
        }
        isProcessing = true;
        worker = new SwingWorker() {

            public Object construct()
            {
                try
                {
                    statusBar.updateStatus("Decoding Page", 0);
                    decode_pdf.addImage(img);
                    if(resizePanel && showThumbnails)
                        zoom();
                    if(Thread.interrupted())
                        throw new InterruptedException();
                    pageCounter2.setForeground(Color.black);
                    pageCounter2.setText(" " + currentPage);
                    pageCounter3.setText(getMessage("PdfViewerOfLabel.text") + " " + pageCount);
                    setViewerTitle(null);
                }
                catch(Exception exception)
                {
                    setViewerTitle(null);
                }
                statusBar.setProgress(100);
                scalingBox.setEnabled(true);
                rotationBox.setEnabled(true);
                return null;
            }

        };
        worker.start();
    }

    private void addExampleListeners()
    {
        String s = System.getProperty("listenForms");
        if(s != null)
        {
            AcroRenderer acrorenderer = decode_pdf.getCurrentFormRenderer();
            java.util.List list = null;
            try
            {
                list = acrorenderer.getComponentNameList(currentPage);
            }
            catch(PdfException pdfexception)
            {
                LogWriter.writeLog("Exception " + pdfexception + " reading component list");
            }
            if(list == null)
            {
                JOptionPane.showMessageDialog(c, "No fields on this page");
                return;
            }
            int i = list.size();
            JPanel jpanel = new JPanel();
            jpanel.setLayout(new BoxLayout(jpanel, 1));
            JLabel jlabel = new JLabel("This page contains " + i + " form objects");
            jlabel.setFont(headFont);
            jpanel.add(jlabel);
            jpanel.add(Box.createRigidArea(new Dimension(10, 10)));
            JTextPane jtextpane = new JTextPane();
            jtextpane.setPreferredSize(new Dimension(450, 180));
            jtextpane.setEditable(false);
            jtextpane.setText("This provides a simple example of Forms handling. We have added a listener to each form so clicking on it shows the form name.\n\nCode is in addExampleListeners() in org.examples.simpleviewer.SimpleViewer\n\nThis could be easily be extended to interface with a database directly or collect results on an action and write back using itext.\n\nForms have been converted into Swing components and are directly accessible (as is the original data).\n\nIf you don't like the standard SwingSet you can replace with your own set.");
            jtextpane.setFont(textFont);
            jpanel.add(jtextpane);
            jpanel.add(Box.createRigidArea(new Dimension(10, 10)));
            for(int j = 0; j < i; j++)
            {
                String s1 = (String)list.get(j);
                java.awt.Component acomponent[] = acrorenderer.getComponentsByName(s1);
                Integer integer = new Integer(j);
                if(acomponent == null || pagesDecoded.get(integer) != null)
                    continue;
                pagesDecoded.put(integer, "x");
                int k = acomponent.length;
                for(int l = 0; l < k; l++)
                {
                    if(l == 0)
                    {
                        JLabel jlabel1 = new JLabel();
                        JLabel jlabel2 = new JLabel("Form name=" + s1);
                        String s2 = "type=" + acomponent[l].getClass();
                        if(k > 1)
                        {
                            s2 = "Group of " + k + " Objects, type=" + acomponent[l].getClass();
                            jlabel1.setForeground(Color.white);
                        }
                        jlabel1.setText(s2);
                        jlabel2.setFont(headFont);
                        jlabel1.setFont(textFont);
                        jpanel.add(jlabel2);
                        jpanel.add(jlabel1);
                        jpanel.add(new JLabel(" "));
                    }
                    if(acomponent[l] instanceof JComboBox)
                    {
                        FormActionListener formactionlistener = new FormActionListener(s1 + l, c);
                        ((JComboBox)acomponent[l]).addActionListener(formactionlistener);
                        continue;
                    }
                    if(acomponent[l] instanceof JCheckBox)
                    {
                        FormActionListener formactionlistener1 = new FormActionListener(s1 + l, c);
                        ((JCheckBox)acomponent[l]).addActionListener(formactionlistener1);
                        continue;
                    }
                    if(acomponent[l] instanceof JRadioButton)
                    {
                        FormActionListener formactionlistener2 = new FormActionListener(s1 + l, c);
                        ((JRadioButton)acomponent[l]).addActionListener(formactionlistener2);
                        continue;
                    }
                    if(acomponent[l] instanceof JTextField)
                    {
                        FormActionListener formactionlistener3 = new FormActionListener(s1 + l, c);
                        ((JTextField)acomponent[l]).addActionListener(formactionlistener3);
                    }
                }

            }

            final JDialog displayFrame = new JDialog(this, true);
            JScrollPane jscrollpane = new JScrollPane();
            jscrollpane.getViewport().add(jpanel);
            jscrollpane.setHorizontalScrollBarPolicy(30);
            jscrollpane.setVerticalScrollBarPolicy(20);
            displayFrame.setLocationRelativeTo(null);
            displayFrame.setLocation(getLocationOnScreen().x + 10, getLocationOnScreen().y + 10);
            displayFrame.setSize(500, 500);
            displayFrame.setTitle("List of forms on this page");
            displayFrame.getContentPane().setLayout(new BorderLayout());
            displayFrame.getContentPane().add(jscrollpane, "Center");
            JPanel jpanel1 = new JPanel();
            jpanel1.setLayout(new BorderLayout());
            displayFrame.getContentPane().add(jpanel1, "South");
            JButton jbutton = new JButton("Close");
            jbutton.setFont(new Font("SansSerif", 0, 12));
            jpanel1.add(jbutton, "East");
            jbutton.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent actionevent)
                {
                    displayFrame.dispose();
                }

            });
            displayFrame.show();
        }
    }

    public void back(int i)
    {
        if(!isProcessing)
        {
            if(currentPage - i >= 1)
            {
                currentPage = currentPage - i;
                statusBar.resetStatus("loading page " + currentPage);
                decode_pdf.setPageParameters(scaling, currentPage);
                decodePage(false);
                if(scalingBox.getSelectedIndex() < 3)
                    zoom();
            }
        } else
        {
            JOptionPane.showMessageDialog(this, getMessage("PdfViewerDecodeWait.message"));
        }
    }

    public void exit()
    {
        if(showThumbnails && thumbnails.drawing)
        {
            thumbnails.interrupt = true;
            while(thumbnails.drawing) 
                try
                {
                    Thread.sleep(20L);
                }
                catch(InterruptedException interruptedexception)
                {
                    interruptedexception.printStackTrace();
                }
            thumbnails.interrupt = false;
        }
        JOptionPane.showConfirmDialog(this, new JLabel(getMessage("PdfViewerExiting")), getMessage("PdfViewerprogramExit"), -1, -1);
        decode_pdf.closePdfFile();
        if(target != null)
            flush();
        System.exit(1);
    }

    protected String getMessage(String s)
    {
        String s1 = null;
        try
        {
            s1 = bundle.getString(s);
        }
        catch(Exception exception) { }
        if(s1 == null)
            try
            {
                if(messages == null)
                    initMessages();
                s1 = (String)messages.get(s);
            }
            catch(Exception exception1) { }
        if(s1 == null)
            s1 = s;
        return s1;
    }

    private void initMessages()
    {
        Object obj = null;
        BufferedReader bufferedreader = null;
        ClassLoader classloader = getClass().getClassLoader();
        String s1 = "Cp1252";
        try
        {
            messages = new HashMap();
            bufferedreader = new BufferedReader(new InputStreamReader(classloader.getResourceAsStream("org/jpedal/international/messages.properties"), s1));
            if(bufferedreader == null)
                LogWriter.writeLog("Unable to open messages.properties from jar");
            do
            {
                String s = bufferedreader.readLine();
                if(s == null)
                    break;
                int i = s.indexOf('=');
                if(i != -1)
                    messages.put(s.substring(0, i), s.substring(i + 1));
            } while(true);
        }
        catch(Exception exception)
        {
            LogWriter.writeLog("Exception " + exception + " reading message Bundle");
        }
        if(bufferedreader != null)
            try
            {
                bufferedreader.close();
            }
            catch(Exception exception1)
            {
                LogWriter.writeLog("Exception " + exception1 + " reading lookup table for pdf  for abobe map");
            }
    }

    protected void selectBookmark()
    {
        if(!decode_pdf.hasOutline())
            return;
        else
            return;
    }

    private void traverse()
    {
        TreeModel treemodel = tree.getModel();
        if(treemodel != null)
            walk(treemodel, treemodel.getRoot());
    }

    private void walk(TreeModel treemodel, Object obj)
    {
        int i = treemodel.getChildCount(obj);
        for(int j = 0; j < i; j++)
        {
            Object obj1 = treemodel.getChild(obj, j);
            if(treemodel.isLeaf(obj1))
            {
                DefaultMutableTreeNode defaultmutabletreenode = (DefaultMutableTreeNode)obj1;
                String s = (String)defaultmutabletreenode.getUserObject();
                String s1 = (String)pageLookupTable.get(s);
                if(s1.length() <= 0)
                    continue;
                try
                {
                    if(Integer.parseInt(s1) == currentPage)
                    {
                        ignoreAlteredBookmark = true;
                        tree.setSelectionPath(new TreePath(defaultmutabletreenode.getPath()));
                        ignoreAlteredBookmark = false;
                    }
                    continue;
                }
                catch(NumberFormatException numberformatexception)
                {
                    System.out.println("bad page number: " + s1);
                }
                ignoreAlteredBookmark = false;
            } else
            {
                walk(treemodel, obj1);
            }
        }

    }

    private void createLookupTable(TreeModel treemodel, Object obj)
    {
        int i = treemodel.getChildCount(obj);
        for(int j = 0; j < i; j++)
        {
            Object obj1 = treemodel.getChild(obj, j);
            if(treemodel.isLeaf(obj1))
            {
                DefaultMutableTreeNode defaultmutabletreenode = (DefaultMutableTreeNode)obj1;
                String s = (String)defaultmutabletreenode.getUserObject();
                String s1 = (String)pageLookupTable.get(s);
                try
                {
                    if(Integer.parseInt(s1) == currentPage)
                        tree.setSelectionPath(new TreePath(defaultmutabletreenode.getPath()));
                }
                catch(NumberFormatException numberformatexception)
                {
                    System.out.println("bad page number: " + s1);
                }
            } else
            {
                walk(treemodel, obj1);
            }
        }

    }

    private void searchText()
        throws Exception
    {
        if(searcher != null)
            searcher.interrupt();
        searchButton.setText("stop");
        searchButton.invalidate();
        searchButton.repaint();
        isSearch = true;
        searchCount.setText("Scanning");
        searchCount.repaint();
        searcher = new SwingWorker() {

            public Object construct()
            {
                try
                {
                    listModel.removeAllElements();
                    results.repaint();
                    int i = 0;
                    textPages.clear();
                    textRectangles.clear();
                    itemFoundCount = 0;
                    decode_pdf.setFoundTextAreas(null);
                    String s = searchText.getText();
                    PdfPageData pdfpagedata = decode_pdf.getPdfPageData();
                    int j1 = 1;
                    int k1 = pageCount + 1;
                    if(!searchAll.isSelected())
                    {
                        j1 = currentPage;
                        k1 = j1 + 1;
                    }
                    for(int l1 = j1; l1 < k1; l1++)
                    {
                        if(Thread.interrupted())
                            throw new InterruptedException();
                        Object obj = null;
                        try
                        {
                            PdfGroupingAlgorithms pdfgroupingalgorithms;
                            if(l1 == currentPage)
                            {
                                pdfgroupingalgorithms = decode_pdf.getGroupingObject();
                            } else
                            {
                                decode_pdf.decodePageInBackground(l1);
                                pdfgroupingalgorithms = decode_pdf.getBackgroundGroupingObject();
                            }
                            pdfgroupingalgorithms.generateTeasers();
                            int j = pdfpagedata.getMediaBoxX(l1);
                            int l = pdfpagedata.getMediaBoxWidth(l1);
                            int k = pdfpagedata.getMediaBoxY(l1);
                            int i1 = pdfpagedata.getMediaBoxHeight(l1);
                            float af[] = pdfgroupingalgorithms.findTextInRectangle(j, i1, l + j, k, l1, s, false, true);
                            float af1[] = pdfgroupingalgorithms.getEndPoints();
                            String as[] = pdfgroupingalgorithms.getTeasers();
                            if(Thread.interrupted())
                                throw new InterruptedException();
                            if(af != null && as != null)
                            {
                                itemFoundCount = itemFoundCount + as.length;
                                int i2 = af.length;
                                int j2 = 0;
                                for(int k2 = 0; k2 < i2; k2 += 2)
                                {
                                    int l2 = (int)af[k2];
                                    int i3 = (int)af[k2 + 1];
                                    int j3 = (int)af1[k2];
                                    int k3 = (int)af1[k2 + 1];
                                    final String tease = as[k2 / 2];
                                    Runnable runnable = new Runnable() {

                                        public void run()
                                        {
                                            listModel.addElement(tease);
                                        }

                                    };
                                    SwingUtilities.invokeAndWait(runnable);
                                    Integer integer = new Integer(i);
                                    i++;
                                    textRectangles.put(integer, new Rectangle(l2, k3, j3 - l2, i3 - k3));
                                    textPages.put(integer, new Integer(l1));
                                    j2++;
                                }

                            }
                            if((af != null) | (l1 % 16 == 0))
                            {
                                searchCount.setText("Items found " + itemFoundCount + " Scanning:" + l1);
                                searchCount.invalidate();
                                searchCount.repaint();
                            }
                        }
                        catch(PdfException pdfexception) { }
                    }

                    searchCount.setText("Items found " + itemFoundCount + " Done");
                    results.invalidate();
                    results.repaint();
                    isSearch = false;
                    searchButton.setText("search");
                }
                catch(Exception exception) { }
                return null;
            }


        };
        searcher.start();
    }

    protected ResourceBundle bundle;
    Map textPages;
    Map textRectangles;
    JTextField searchText;
    JCheckBox searchAll;
    JTextField searchCount;
    DefaultListModel listModel;
    SearchList results;
    boolean display;
    protected boolean switchModes;
    private boolean showThumbnailsdefault;
    private boolean showThumbnails;
    public ThumbnailPanel thumbnails;
    final boolean extractImageOnSelection = true;
    protected final String separator = System.getProperty("file.separator");
    protected PdfDecoder decode_pdf;
    protected PdfAnnots pageAnnotations;
    protected JScrollPane bookmarkScrollPane;
    protected JTabbedPane navOptionsPanel;
    protected JSplitPane displayPane;
    private JTree tree;
    protected Container c;
    static SimpleViewer current;
    private boolean isPDF;
    JFrame searchFrame;
    private boolean firstTime;
    protected String titleMessage;
    private HashMap pageLookupTable;
    private Map pointLookupTable;
    private boolean ignoreAlteredBookmark;
    private String inputDir;
    protected int currentPage;
    protected String target;
    protected int rotation;
    public int m_x1;
    public int m_y1;
    public int m_x2;
    public int m_y2;
    public int old_m_x2;
    public int old_m_y2;
    protected int x;
    protected int y;
    public Rectangle currentRectangle;
    public int cx;
    public int cy;
    protected static int printingThreads = 0;
    SwingWorker worker;
    SwingWorker searcher;
    public boolean isSearch;
    JButton searchButton;
    private static JLabel coords = new JLabel();
    protected int pageCount;
    protected float scaling;
    protected String selectedFile;
    protected boolean isProcessing;
    protected String scalingValues[] = {
        "25", "50", "75", "100", "125", "150", "200", "250", "500", "750", 
        "1000"
    };
    protected float scalingFloatValues[] = {
        0.25F, 0.5F, 0.75F, 1.0F, 1.25F, 1.5F, 2.0F, 2.5F, 5F, 7.5F, 
        10F
    };
    private final int defaultSelection = 0;
    protected JComboBox scalingBox;
    JScrollPane scrollPane;
    private final String rotationValues[] = {
        "0", "90", "180", "270"
    };
    protected JComboBox rotationBox;
    protected JLabel pageCounter1;
    protected JTextField pageCounter2;
    protected JLabel pageCounter3;
    private Map messages;
    private BufferedImage img;
    private final int inset = 25;
    private long size;
    private String message;
    private static int currentFileCount = 0;
    protected JButton first;
    protected JButton fback;
    protected JButton back;
    protected JButton forward;
    protected JButton fforward;
    protected JButton end;
    private final String annotTypes[] = {
        "Other", "Text", "FileAttachment"
    };
    private boolean allowScrolling;
    private boolean hasOutlinesDrawn;
    private JButton bookmarksButton;
    private int cropX;
    private int cropY;
    private int cropW;
    private int cropH;
    private int mediaW;
    private int mediaH;
    private int mediaX;
    private int mediaY;
    private final Font textFont = new Font("Serif", 0, 12);
    private final Font headFont = new Font("SansSerif", 1, 14);
    protected StatusBar statusBar;
    private String ocr[] = {
        "TeleForm", "dgn2pdf"
    };
    public int divLocation;
    private int thumbLocation;
    private int minimumScreenWidth;
    private int dx;
    private int dy;
    private double viewportScale;
    protected int maxViewY;
    private ThumbPainter painter;
    protected boolean showOutlines;
    private int itemFoundCount;
    private HashMap pagesDecoded;







































}
